# -*- coding: utf-8 -*-
=begin
  Copyright (C) 2012-2016 Takashi SUGA

  You may use and/or modify this file according to the license described in the LICENSE.txt file included in this archive.
=end

module When::Coordinates

  #
  # 九星
  #
  class Kyusei < Residue

    StartOfSequence = {-1=>'九星陽遁始め', +1=>'九星陰遁始め'}

    Patch = {2421371=>2421431, 2429771=>2429831, 2463251=>2463191, 2475851=>2475791}

    class << self

      #
      # 年の九星のインデックス
      #
      # @param [Integer] y 西暦の年数
      #
      # @return [Integer]
      #
      def year(y)
        (y + 2) % 9
      end

      #
      # 月の九星のインデックス
      #
      # @param [Integer] m 通月
      #
      # @return [Integer]
      #
      def month(m)
        (m + 2) % 9
      end

      #
      # 日の九星のインデックス
      #
      # @param [When::TM::TemporalPosition] date 節月による日付
      # @param [When::CalendarNote::Japanese::SolarTerms] s_terms
      #
      # @return [Integer]
      #
      def day(date, s_terms)
        thres, delta, sign = _parameters(date, s_terms)
        delta += 3 if thres == 30
        delta = 8 - delta if sign < 0
        return delta % 9
      end

      #
      # 陰遁・陽遁折り返し日か?
      #
      # @param [When::TM::TemporalPosition] date 節月による日付
      # @param [When::CalendarNote::Japanese::SolarTerms] s_terms
      #
      # @return [String]
      #
      def is_turn?(date, s_terms)
        thres, delta, sign = _parameters(date, s_terms)
        return nil unless thres == delta
        StartOfSequence[sign]
      end

      private

      #
      # 日の九星計算パラメータ
      #
      # @param [When::TM::TemporalPosition] date 節月による日付
      # @param [When::CalendarNote::Japanese::SolarTerms] s_terms
      #
      # @return [Integer, Integer, Integer]
      #
      def _parameters(date, s_terms)
        date = When.when?(date.to_cal_date.to_s, {:frame=>date.frame, :clock=>date.frame.time_basis})

        bases = [+450,+270,+90,-90,-270,-450].map { |deg|
          prev   = s_terms.term(date, [deg,180])              # 直後および直前の冬至または夏至
          sign   = s_terms.position(prev)[0] == 270 ? -1 : +1 # 陽遁か?(冬至:-1, 夏至:+1)
          base   = prev.to_i                                  # 当該の冬至または夏至のユリウス日
          kanshi = (base + 17) % 60 - 28                      # 最も近い甲子との日数差
          base  -= kanshi                                     # 当該陰陽遁の始めの甲子
          [(Patch[base] || base), sign]                       # 『暦日大鑑』に合わせる
        }

        (1...bases.size).each do |i|                          # 区間が短くなりすぎる場合は次と繋げる
          next unless bases[i-1][0] - bases[i][0] == 120
          bases[i-1][0] += 60
          break
        end

        bases.size.times do |i|
          base, sign = bases[i]
          delta  = date.to_i - base                           # 始めからの日数
          next if delta < 0
          case (bases[i-1][0] - base)
          when 180
            return [ 0, delta, sign]                          # 通常
          when 240
            return [30, delta,      sign] if delta >= 30      # 甲午以降
            return [ 0, delta+180, -sign]                     # 甲午より前
          else
            raise ArgumentError, "Invalid solstice interval: #{bases[i-1][0] - base}"
          end
       end
       raise ArgumentError, "can't find any solstice."
      end
    end
  end

  #
  # 十干
  #
  class Stem < Residue

    Rules = {
      '歳徳' => %w(東宮甲 西宮庚 南宮丙 北宮壬 南宮丙 東宮甲 西宮庚 南宮丙 北宮壬 南宮丙),
      '金神' => %w(午未申酉 辰巳 子丑寅卯午未 寅卯戌亥 子丑申酉)
    }

    #
    # 年の十干で決まる暦注
    # @private
    def _year_notes(notes={}, dates=nil, conditions={})
      notes['歳徳']     ||= Rules['歳徳'][@remainder]
      notes['金神']     ||= Rules['金神'][@remainder % 5]
      notes
    end

    #
    # 月の十干で決まる暦注(ダミー)
    # @private
    def _month_notes(notes={}, dates=nil, conditions={})
      notes
    end

    #
    # 日の十干で決まる暦注
    # @private
    def _day_notes(notes={}, dates=nil, conditions={})

      year, month = dates.s_date.cal_date

      # 節月に依存する暦注
      notes['復' ]      ||= [[4,5],[0,6],[1,7],[4,5],[2,8],[3,9]][month % 6].include?(@remainder) ? '復' : nil

      # 不断の暦注
      notes['不視病']   ||= @remainder == 4 ? '不視病' : nil
      notes['不問疾']   ||= @remainder == 5 ? '不問疾' : nil

      # 凶会日に省略される吉日
      return notes if notes['凶会']

      # 節年に依存する暦注
      notes['歳徳'  ]   ||= @remainder == [6,2,8,4,0][year % 5] ? '歳徳'   : nil
      notes['歳徳'  ]   ||= @remainder == [1,7,3,9,5][year % 5] ? '歳徳合' : nil

      # 節月に依存する暦注
      notes['月徳'  ]   ||= @remainder == [6,2,0,8][month % 4]  ? '月徳'   : nil
      notes['月徳'  ]   ||= @remainder == [1,7,5,3][month % 4]  ? '月徳合' : nil
      notes
    end
  end


  #
  # 十二支
  #
  class Branch < Residue

    Rules = {
      '大將軍' => %w(酉 酉 子 子 子 卯 卯 卯 午 午 午 酉),
      '大陰'   => %w(戌 亥 子 丑 寅 卯 辰 巳 午 未 申 酉),

      '歳刑'   => %w(卯 戌 巳 子 辰 申 午 丑 寅 酉 未 亥),
      '破'     => %w(午 未 申 酉 戌 亥 子 丑 寅 卯 辰 巳),
      '煞'     => %w(未 辰 丑 戌),
      '黄幡'   => %w(辰 丑 戌 未 辰 丑 戌 未 辰 丑 戌 未),
      '豹尾'   => %w(戌 未 辰 丑 戌 未 辰 丑 戌 未 辰 丑),
      '歳次'   => %w(玄枵 星紀 析木 大火 壽星 鶉尾 鶉火 鶉首 實沈 大梁 降婁 娵訾),

      '天氣'   => %w(東南 西 南 西南 北 西 西北 東 北 東北 南 東),
      '天道'   => %w(艮坤 甲庚 乙辛 巽乾 丙壬 丁癸),
      '人道'   => %w(巽乾 丙壬 丁癸 艮坤 甲庚 乙辛),
      '天徳'   => %w(巽 庚 丁 坤 壬 辛 乾 甲 癸 艮 丙 乙),
      '用時'   => %w(艮巽坤乾 乙丁辛亥 甲丙庚壬),
      '月徳'   => %w(壬 庚 丙 甲),
      '月徳合' => %w(丁 乙 辛 己),
      '月空'   => %w(丙 甲 壬 庚),
      '三鏡'   => %w(甲丙庚壬坤乾 甲乙丁庚辛癸 乙辛乾坤巽艮 甲丙庚壬乾巽
	             乙丙丁辛壬癸 丁癸乾坤巽艮 甲丙庚壬坤艮 甲乙丁庚辛癸
	             乙辛乾艮巽坤 甲丙庚壬乾巽 乙丙丁辛壬癸 丁癸坤乾艮巽),
      '土府'   => %w(申 子 丑 巳 酉 寅 午 戌 卯 未 亥 辰),
      '土公'   => %w(庭 庭 竈 竈 竈 門 門 門 井 井 井 庭),

      '十二直' => %w(建 除 滿 平 定 執 破 危 成 收 開 閉),
      '三禍'   => %w(子 丑 寅 卯 辰 巳 午 未 申 酉 戌 亥),
      '下食時' => %w(亥+ 子+ 丑+ 寅+ 卯 辰 巳 午 未 申 酉+ 戌+)
    }

    #
    # 年の十二支で決まる暦注
    # @private
    def _year_notes(notes={}, dates=nil, conditions={})
      notes['大將軍']   ||= Rules['大將軍'][@remainder]
      notes['大陰'  ]   ||= Rules['大陰'  ][@remainder]
      notes['歳刑'  ]   ||= Rules['歳刑'  ][@remainder]
      notes['歳破'  ]   ||= Rules['破'    ][@remainder]
      notes['歳煞'  ]   ||= Rules['煞'    ][@remainder % 4]
      notes['黄幡'  ]   ||= Rules['黄幡'  ][@remainder]
      notes['豹尾'  ]   ||= Rules['豹尾'  ][@remainder]
      if dates.range < 2
        notes['天道']   ||= Rules['天道'  ][@remainder % 6]
        notes['人道']   ||= Rules['人道'  ][@remainder % 6]
      else
        notes['歳次']   ||= Rules['歳次'  ][@remainder]
      end
      notes
    end

    #
    # 月の十二支で決まる暦注
    # @private
    def _month_notes(notes={}, dates=nil, conditions={})
      if dates.range < 2
        notes['天氣']   ||= Rules['天氣'  ][@remainder] + '行'
        notes['天道']   ||= Rules['天道'  ][@remainder % 6]
        notes['人道']   ||= Rules['人道'  ][@remainder % 6]
        notes['月破']   ||= Rules['破'    ][@remainder]
      else
        notes['天道']   ||= Rules['天氣'  ][@remainder] + '行'
      end
      notes['天徳'  ]   ||= Rules['天徳'  ][@remainder]
      notes['月煞'  ]   ||= Rules['煞'    ][@remainder % 4]
      notes['用時'  ]   ||= Rules['用時'  ][@remainder % 3]
      notes['月徳'  ]   ||= Rules['月徳'  ][@remainder % 4]
      notes['月徳合']   ||= Rules['月徳合'][@remainder % 4]
      notes['月空'  ]   ||= Rules['月空'  ][@remainder % 4]
      notes['三鏡'  ]   ||= Rules['三鏡'  ][@remainder]
      notes['土府'  ]   ||= Rules['土府'  ][@remainder]
      notes['土公'  ]   ||= Rules['土公'  ][@remainder]
      notes
    end

    #
    # 日の十二支で決まる暦注
    # @private
    def _day_notes(notes={}, dates=nil, conditions={})

      # 節月
      month = dates.s_date.cal_date[1]
      born  = Rules['三禍'][month-1] + '年'

      # 具注暦(元嘉暦以来)
      notes['十二直']   ||= Rules['十二直'][(@remainder-month-1) % 12]
      notes['歸忌'  ]   ||= @remainder == (month % 3) ? '歸忌' : nil
      notes['血忌'  ]   ||= @remainder == [1,7,2,8,3,9,4,10,5,11,6,0][month-1] ? '血忌' : nil
      unless notes['九坎'] && notes['小字注']
        notes['九坎'  ]   = @remainder == [4,1,10,7,3,0,9,6,2,11,8,5][month-1] ? '九坎' : nil
      end
      notes['厭'    ]   ||= @remainder == (17-month) % 12 ? '厭對' : nil
      notes['厭'    ]   ||= @remainder == (11-month) % 12 ? '厭'   : nil
      notes['天倉'  ]   ||= @remainder == (3-month) % 12  ? '天倉' : nil

      # 具注暦(大衍暦以来)
      notes['忌遠行']   ||= @remainder == [1,9,5][month % 3]            ? '忌遠行' : nil
      notes['忌夜行']   ||= @remainder == [0,6,5,10,7,4][(month-1) / 2] ? '忌夜行' : nil
      notes['大禍'  ]   ||= @remainder == (month * 7 +  4) % 12 ?  born + '大禍' : nil
      notes['狼藉'  ]   ||= @remainder == (conditions[:kana] ? ((month-1) % 4) * 3 :                      #  - wikipedia 「天火」におなじ
                                               [0,3,6,6,0,3,9,9,3,3,6,9][month-1]) ?  born + '狼藉' : nil # %w(...酉..午.子...)
      notes['滅門'  ]   ||= @remainder == (month * 7 + 10) % 12 ?  born + '滅門' : nil
      notes['下食時']   ||= @remainder == [7,10,4,2,6,0,8,9,5,11,1,3][month-1] ? Rules['下食時'][month-1] : nil
      notes['下食時']     = dates.range < 11 ? notes['下食時' ].sub(/\+/, '') : nil if /\+/ =~ notes['下食時'].to_s # '+' は貞享改暦以降記さない
      notes['無翹'  ]   ||= @remainder == (12-month) % 12 ? '無翹' : nil
      notes['不弔人']   ||= [4,6].include?(@remainder) ? '不弔人' : nil # 不断
      notes['重'    ]   ||= @remainder % 6 == 5 ? '重' : nil            # 不断
      unless notes['凶会'] || notes['没'] || notes['滅']
        notes['沐浴']     ||= [0,8,9,11].include?(@remainder) ? '沐浴' : nil
      end
      notes['月煞'  ]   ||= @remainder == [4,1,10,7][month % 4] ? '月煞' : nil

      # 仮名暦
      notes['初午']     ||= @remainder == 6 && dates.o_date.cal_date[1] == 2 && (1..12).include?(dates.o_date.cal_date[2]) ? '初午' : nil
      notes['天火'  ]   ||= @remainder == ((month-1) % 4) * 3 ? '天火' : nil
      notes['地火'  ]   ||= @remainder == (month + 4) % 12 ? '地火' : nil
      notes['人火'  ]   ||= @remainder == [12,0,8,10,12,3,7,6,12,6,9,9][month-1] ? '人火' : nil
      notes['雷火'  ]   ||= @remainder == [12,7,7,5,12,6,6,5,12,11,7,6][month-1] ? '雷火' : nil
      notes['一粒万倍'] ||= [[1,6],[2,9],[0,3],[3,4 ],[5,6 ],[6,9],
	                     [0,7],[3,8],[6,9],[9,10],[0,11],[0,3]][month-1].include?(@remainder) ? '一粒万倍' : nil
      notes['金神間日'] ||= @remainder == [1,8,7,9][(month-1) / 3] ? '金神間日' : nil
      notes['十死'    ] ||= @remainder == [1,9,5][month % 3] ? '十死' : nil
      notes['受死'    ] ||= @remainder == [10,4,11,5,0,6,1,7,2,8,3,9][month-1] ? '受死' : nil
      notes['天福'    ] ||= @remainder == [5,2,11,8][month % 4] ? '天福' : nil
      notes['地福'    ] ||= @remainder == [11,8,5,2][month % 4] ? '地福' : nil
      notes['地五福'  ] ||= @remainder == [8,11,2,5][month % 4] ? '地五福' : nil
      notes['三隣亡'  ] ||= @remainder == [6,11,2][month % 3] ? '三隣亡' : nil
      notes['金性']     ||= @remainder == 3 && month == 2  && dates.s_date.cal_date[0] % 12 == 7 &&
                                             (1..12).include?(dates.s_date.cal_date[2]) ? '金性' : nil

      notes
    end
  end

  #
  # 六十干支
  #
  class StemBranch < Residue

    #
    # 年の六十干支で決まる暦注
    # @private
    def _year_notes(notes={}, dates=nil, conditions={})
      unless dates.range < 2
        notes['大歳壇'] ||= '干' + Rules['大歳壇干'][(@remainder / 2) % 5] + '支' + Rules['大歳壇支'][@remainder % 12]
      end
      notes['納音']     ||= Rules['納音'][@remainder / 2]
      notes['大歳']     ||= self
      notes
    end

    #
    # 月の六十干支で決まる暦注
    # @private
    def _month_notes(notes={}, dates=nil, conditions={})
      notes['月建']     ||= self
      notes
    end

    #
    # 日の六十干支で決まる暦注
    # @private
    def _day_notes(notes={}, dates=nil, conditions={})

      year, month = dates.s_date.cal_date

      # 具注暦(元嘉暦以来)
      notes['納音']     ||= Rules['納音'][@remainder / 2]
      notes['日遊']     ||= dates.range < 2 ? Rules['日遊'][@remainder] :
                            [4,5,14,15,24,25,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,54,55].include?(@remainder) ? '内' : nil
      notes['天赦']     ||= @remainder == [14,30,44,00][(month-1)/3] ? '天赦' : nil

      #   宣明暦以前は節月、貞享暦以後は暦月
    # m  = dates.range < 11 ? month : (dates.m_date.cal_date[1] * 1)
      m  = month
      #   凶会日
      kue = Rules['凶会'][m-1][@remainder]
      #   貞享暦で廃止された凶会日を除外
      kue = dates.range < 11 ? kue.sub(/\+/, '') : nil if /\+/ =~ kue
      notes['凶会'] = kue

      #   凶会日に省略される吉日
      _notes_without_kue(notes, month, dates.range < 2 ? '歳' : '大歳', conditions) unless notes['凶会']

      # 具注暦(大衍暦以来)
      notes['天一']     ||= begin
        note = Rules['天一'][@remainder]
        dates.range >= 15 ?  note.dup.sub!(/天上始/, '天上') :
        dates.range >= 10 ?  note : note.dup.sub!(/始/, '')
      end
      notes['土公']     ||= Rules['土公'][@remainder]
      notes['五墓']     ||= [4,22,28,31,37].include?(@remainder) ? '五墓' : nil
      notes['伐'  ]     ||= [6,12,14,15,17,19,20,21,23,28,49,58].include?(@remainder) ? '伐' : nil

      #   節年に依存する暦注
      div, mod = @remainder.divmod(12)
      base_dir = Rules['環'][year % 12]
      move_dir = Rules['遊'][div]
      unless base_dir == move_dir
	event  = '遊' + move_dir if mod == 0
	event  = '環' + base_dir if mod == 5
      end
      notes['大將軍']   ||= event
      notes['歳下食']   ||= @remainder == [33,22,47,36,13,26,3,28,53,42,43,56][year % 12] ? '歳下食'  : nil

      #   節月に依存する暦注
      notes['天間']     ||= (@remainder % 30) == [23,0,1,2,3,22][(month-1) % 6] ? '天間' : nil
      notes['八龍']     ||= (month-1) / 3 == 0 && [ 0,11].include?(@remainder)  ? '八龍' : nil
      notes['七鳥']     ||= (month-1) / 3 == 1 && [12,23].include?(@remainder)  ? '七鳥' : nil
      notes['九虎']     ||= (month-1) / 3 == 2 && [36,47].include?(@remainder)  ? '九虎' : nil
      notes['六蛇']     ||= (month-1) / 3 == 3 && [48,59].include?(@remainder)  ? '六蛇' : nil

      #   宣明暦以前は節月、貞享暦以後は暦月？
      notes['三寶吉'], notes['神吉'], notes['雑事吉'], notes['小字注'], kana = Rules['吉事注'][m-1][@remainder]
      if conditions[:kana]
        if kana && dates.m_date.ceil(YEAR,MONTH).prev != dates.m_date # 最後の月以外は年末行事を削除する
          kana = kana.dup
          kana.sub!(/すゝはらひ\s?/, '')
          kana.sub!(/正月ことはしめ\s?/, '')
          kana.strip!
          kana = nil if kana == ''
        end
        notes['小字注']   = kana
        notes['神吉']     = '(神吉)' if notes['神吉'] && Rules['神吉'][@remainder][12-month] == 0
      end

      # 仮名暦
    # notes['大明']     ||= [5,6,7,8,9,13,15,18,20,23,28,31,38,40,41,42,43,45,46,47,52,54,55,56,57].include?(@remainder) ? '大明' : nil
      notes['大明']     ||= [    7,8,9,13,15,18,20,23,28,31,38,40,41,42,   45,46,47,52,   55,56,57].include?(@remainder) ? '大明' : nil
      unless notes['金神遊行']
        div, mod   = @remainder.divmod(12)
        note       = (2..6).include?(mod) ? '平素' + Rules['平素遊行'][div] : ''
        range, dir = Rules['四季遊行'][(month-1) / 3]
        note      += range.include?(@remainder) ? '四季' + dir : ''
        notes['金神遊行'] = note == '' ? nil : note
      end
      notes['甲子待']   ||= @remainder == 0  ? '甲子待' : nil
      notes['己巳'  ]   ||= @remainder == 5  ? '己巳'   : nil
      notes['庚申待']   ||= @remainder == 56 ? '庚申待' : nil
      if dates.range >= 14
        notes['犯土']   ||= {6 =>'大犯土始', 14=>'小犯土始'}[@remainder]
      else
        notes['犯土']   ||= [7,8,9,10,11].include?(@remainder)    ? '大犯土' : {6 =>'大犯土始', 12=>'大犯土終', 13=>'犯土間日'}[@remainder]
        notes['犯土']   ||= [15,16,17,18,19].include?(@remainder) ? '小犯土' : {14=>'小犯土始', 20=>'小犯土終'}[@remainder]
      end
      notes['十方暮']   ||= [21,22,23,24,25,26,27,28].include?(@remainder) ? '十方暮' : {20=>'十方暮始', 29=>'十方暮終'}[@remainder]
      unless notes['八專']
        note = Rules['八專'][@remainder]
        note = note.sub('始','入') if note && (9..10).include?(dates.range)
        notes['八專'] = note
      end
      notes['八專間日'] ||= [49,52,54,58].include?(@remainder) ? '八專間日' : nil
    end

    private

    def _notes_without_kue(notes, month, daisai, conditions)
      notes['天恩']     ||= [0,3,9].include?(@remainder / 5) ? '天恩' : nil

      unless notes['除手足甲'] || notes['没'] || notes['滅']
        note  = {1=>'除手甲', 2=>'除足甲'}[@remainder % 12]
        valid = conditions[:kana] ? Rules['除手足甲'][@remainder][12-month] == 1 : true
        notes['除手足甲'] = valid ? note : '(' + note + ')' if note
      end

      unless notes['母倉']
        note1 = [[0,11],    [0,11],    [5,6], [2,3], [2,3], [5,6],
                 [1,4,7,10],[1,4,7,10],[5,6], [8,9], [8,9], [5,6]][month-1].include?(@remainder % 12)
        note2 = conditions[:kana] ? Rules['母倉'][@remainder][12-month] == 1 : note1
        notes['母倉'] =
          case [note1,note2]
          when [true,  true ]  ;  '母倉'
          when [false, true ]  ; '[母倉]'
          when [true,  false]  ; '(母倉)'
          else                 ; nil
          end
      end

      return notes if notes['大小歳']

      if conditions[:sai] && conditions[:sai].to_s != '0'
        note = Rules['大小歳差分'][[month, @remainder]]
        if note
          notes['大小歳'] = daisai + note
          return
        end
      end

      index = Rules['大小歳'][0].map {|pattern|
	        k = 4.times {|i|
	          break i if Rules['大小歳'][1][@remainder] =~ pattern[i]
	        }
	        k == 4 ? nil : Rules['大小歳'][2][(k+(month-1)/3) % 4]
	      }
      notes['大小歳'] = (index[0] ? '大小歳' + index[0] : '') + 
	                (index[1] ?  daisai  + index[1] : '') + 
	                (index[2] ? '小歳'   + index[2] : '')
      notes.delete('大小歳') if notes['大小歳'] == ''
    end

    Rules = {
      '大歳壇干' => %w(木 火 土 金 水),
      '大歳壇支' => %w(水 土 木 木 土 火 火 土 金 金 土 水),
      '納音'     => %w(海中金 爐中火 大林木 路傍土 釼鋒金 山頭火 澗下水 城頭土 白鑞金 楊柳木
	               井泉水 屋上土 霹靂火 松柏木 長流水 沙中金 山下火 平地木 壁上土 金箔金
	               覆燈火 天河水 大駅土 釵釧金 桑柘木 大溪水 沙中土 天上火 柘榴木 大海水),
      '日遊'     => {2=>'離', 7=>'坤', 13=>'兌', 18=>'乾', 24=>'坎', 29=>'中宮', 45=>'艮', 51=>'震', 56=>'巽'},
      '天一'     => %w(辰巳   辰巳終 午始   午     午     午     午終   未申始 未申   未申
	               未申   未申   未申終 酉始   酉     酉     酉     酉終   戌亥始 戌亥 
	               戌亥   戌亥   戌亥   戌亥終 子始   子     子     子     子終   天上始
	               天上   天上   天上   天上   天上   天上   天上   天上   天上   天上
	               天上   天上   天上   天上   天上終 丑寅始 丑寅   丑寅   丑寅   丑寅 
	               丑寅終 卯始   卯     卯     卯     卯終   辰巳始 辰巳   辰巳   辰巳),
      '土公'     => {0=>'遊北', 6=>'入', 14=>'遊東',20=>'入', 30=>'遊南', 36=>'入', 44=>'遊西', 50=>'入'},
=begin
      '八專'     => {48=>'八專始 炎魔天歓喜会', 49=>'八專間日',          50=>'八專 地天歓喜会',
	             51=>'八專 水天般若会',     52=>'八專間日',          53=>'八專 火天諸天会',
	             54=>'八專間日',            55=>'八專 羅刹天不動会', 56=>'八專 風天歓喜仁王会',
	             57=>'八專 吉祥天豊楽会',   58=>'八專間日',          59=>'八專終 多聞天成仏会'},
=end
      '八專'     => {48=>'八專始', 59=>'八專終'},
      '平素遊行' => %w(西 中央 北 東 南),
      '四季遊行' => [[[51,52,53,54,55], '東'],
		     [[42,43,44,45,46], '南'],
		     [[57,58,59, 0, 1], '西'],
		     [[48,49,50,51,52], '北']],
      '環'       => %w(南 南 南 西 西 西 北 北 北 東 東 東),
      '遊'       => %w(東 南 内 西 北),
      '大小歳'   => [[[/◇△/,    /■▼/,    /◆▲/,    /□▽/   ],
	              [/◇[^△]/, /■[^▼]/, /◆[^▲]/, /□[^▽]/],
	              [/[^◇]△/, /[^■]▼/, /[^◆]▲/, /[^□]▽/]],
	             %w(◇△ ◇△ ◇△ ◇△ ◇▽ ◇― ◇― ◇― ◇― ◇―
	                ◆▲ ◆▲ ◆▲ ◆▲ ◆▲ ◆― ◇― ◇― ◇― ◇―
	                ◇― ◆▼ ◆▼ ◆▼ ◆― ◆― ◆― □▲ □▲ □▲
	                □▲ □▲ □▲ □▲ □▼ □― □― □― □― □―
	                □― □― ■△ ■△ ■△ ■― ■― ■― □― □―
	                □― □― □― ■▽ ■― ■― ■― ■― ■― ◇△),
	             %w(位 後 對 前)],
      '大小歳差分' => {
                     [5,23] => '前',
                     [5,53] => '對',
                     [8,53] => '前小歳位',
                     [9,53] => '前小歳位'},

      #
      # 神吉 (仮名暦)
      #
      '神吉' => [
        # 123456789XII     123456789XII     123456789XII     123456789XII    # ←節月 ↓日の干支
	0b000000000000,  0b000010010000,  0b000000000000,  0b000101101000,   # 甲子 乙丑 丙寅 丁卯
	0b000000000000,  0b000000100000,  0b100000000000,  0b000000000000,   # 戊辰 己巳 庚午 辛未
	0b110000110001,  0b000010001010,  0b000000000000,  0b000000000000,   # 壬申 癸酉 甲戌 乙亥
	0b000000000000,  0b000010010000,  0b000000000000,  0b000000000000,   # 丙子 丁丑 戊寅 己卯
	0b000000000000,  0b000000000000,  0b110100001100,  0b000000000000,   # 庚辰 辛巳 壬午 癸未
	0b000000000000,  0b000000001000,  0b000000000000,  0b000000000000,   # 甲申 乙酉 丙戌 丁亥
	0b000000000000,  0b000000000000,  0b000000000000,  0b001001000101,   # 戊子 己丑 庚寅 辛卯
	0b000000000000,  0b000000000000,  0b100000100101,  0b000000000000,   # 壬辰 癸巳 甲午 乙未
	0b010000110001,  0b000000000000,  0b000000000000,  0b110000000000,   # 丙申 丁酉 戊戌 己亥
	0b111000000101,  0b000010010000,  0b000000000000,  0b101101101101,   # 庚子 辛丑 壬寅 癸卯
	0b000000000000,  0b000000101000,  0b110000101101,  0b010000000010,   # 甲辰 乙巳 丙午 丁未
	0b010000000001,  0b001010001000,  0b000000000000,  0b010100000010,   # 戊申 己酉 庚戌 辛亥
	0b111000101000,  0b000000000000,  0b000000000000,  0b000101101101,   # 壬子 癸丑 甲寅 乙卯
	0b000000000000,  0b000000000000,  0b110000100101,  0b011000000011,   # 丙辰 丁巳 戊午 己未
	0b100000010001,  0b000010001010,  0b000000000000,  0b000000000000],  # 庚申 辛酉 壬戌 癸亥

      #
      # 母倉 (仮名暦)
      #
      '母倉' => [
        # 123456789XII     123456789XII     123456789XII     123456789XII    # ←節月 ↓日の干支
	0b110000000000,  0b000000010000,  0b000110000000,  0b000110000000,   # 甲子 乙丑 丙寅 丁卯
	0b000000110000,  0b001000001001,  0b001000001001,  0b000000100000,   # 戊辰 己巳 庚午 辛未
	0b000000000010,  0b000000000010,  0b000000110000,  0b110000000000,   # 壬申 癸酉 甲戌 乙亥
	0b110000000000,  0b000000010000,  0b000110000000,  0b000110000000,   # 丙子 丁丑 戊寅 己卯
	0b000000110000,  0b101001001001,  0b000000001001,  0b000000100000,   # 庚辰 辛巳 壬午 癸未
	0b000000000010,  0b000000000010,  0b000000111000,  0b110000000000,   # 甲申 乙酉 丙戌 丁亥
	0b111000000000,  0b000000010000,  0b000111000000,  0b000111000000,   # 戊子 己丑 庚寅 辛卯
	0b000000010000,  0b000000000000,  0b001000000000,  0b000000100000,   # 壬辰 癸巳 甲午 乙未
	0b000000000011,  0b000000000010,  0b000000110000,  0b110000000000,   # 丙申 丁酉 戊戌 己亥
	0b010000000000,  0b000000010000,  0b000110000000,  0b000110000000,   # 庚子 辛丑 壬寅 癸卯
	0b000000010000,  0b001001001001,  0b001000001001,  0b000000100000,   # 甲辰 乙巳 丙午 丁未
	0b000000000010,  0b000000000010,  0b000000110000,  0b110000000000,   # 戊申 己酉 庚戌 辛亥
	0b111000000000,  0b000000010000,  0b000111000000,  0b000111000000,   # 壬子 癸丑 甲寅 乙卯
	0b000000111000,  0b000000000000,  0b000000000000,  0b000000101000,   # 丙辰 丁巳 戊午 己未
	0b000000000010,  0b000000000011,  0b000000111000,  0b110000000000],  # 庚申 辛酉 壬戌 癸亥

      #
      # 除手足甲 (仮名暦)
      #
      '除手足甲' => [
        # 123456789XII     123456789XII     123456789XII     123456789XII    # ←節月 ↓日の干支
	0b000000000000,  0b010110010010,  0b110111110111,  0b000000000000,   # 甲子 乙丑 丙寅 丁卯
	0b000000000000,  0b000000000000,  0b000000000000,  0b000000000000,   # 戊辰 己巳 庚午 辛未
	0b000000000000,  0b000000000000,  0b000000000000,  0b000000000000,   # 壬申 癸酉 甲戌 乙亥
	0b000000000000,  0b000110010010,  0b010000000010,  0b000000000000,   # 丙子 丁丑 戊寅 己卯
	0b000000000000,  0b000000000000,  0b000000000000,  0b000000000000,   # 庚辰 辛巳 壬午 癸未
	0b000000000000,  0b000000000000,  0b000000000000,  0b000000000000,   # 甲申 乙酉 丙戌 丁亥
	0b000000000000,  0b010010010010,  0b110111010111,  0b000000000000,   # 戊子 己丑 庚寅 辛卯
	0b000000000000,  0b000000000000,  0b000000000000,  0b000000000000,   # 壬辰 癸巳 甲午 乙未
	0b000000000000,  0b000000000000,  0b000000000000,  0b000000000000,   # 丙申 丁酉 戊戌 己亥
	0b000000000000,  0b000000000000,  0b100111110111,  0b000000000000,   # 庚子 辛丑 壬寅 癸卯
	0b000000000000,  0b000000000000,  0b000000000000,  0b000000000000,   # 甲辰 乙巳 丙午 丁未
	0b000000000000,  0b000000000000,  0b000000000000,  0b000000000000,   # 戊申 己酉 庚戌 辛亥
	0b000000000000,  0b000000000000,  0b010111100110,  0b000000000000,   # 壬子 癸丑 甲寅 乙卯
	0b000000000000,  0b000000000000,  0b000000000000,  0b000000000000,   # 丙辰 丁巳 戊午 己未
	0b000000000000,  0b000000000000,  0b000000000000,  0b000000000000],  # 庚申 辛酉 壬戌 癸亥

      #
      # 凶会日 '+' は貞享暦で廃止
      #
      '凶会' => [
	{  27=>'三陰',     46=>'陰錯+',     50=>'陽錯'},                                  # 正月
	{  15=>'陰道衝陽', 51=>'陽錯',      57=>'陽錯'},                                  #  2月
	{   0=>'絶陰',      1=>'絶陰',       2=>'絶陰',      3=>'絶陰',       4=>'單陰',  #  3月
	    8=>'孤辰',     16=>'陰位',      20=>'行很',     32=>'了戻',      40=>'陽錯',
	   44=>'孤辰',     56=>'陰錯孤辰',  59=>'絶陰+'},
	{   4=>'絶陰',      5=>'陽錯純陽+',  7=>'孤辰',     19=>'孤辰',      31=>'行很',  #  4月
	   35=>'陰陽衝破', 42=>'歳博',      43=>'陰錯了戻', 53=>'陽錯+',     54=>'歳博',
	   55=>'陰錯孤辰', 59=>'陰陽交破'},
	{  42=>'陰陽倶錯', 48=>'陰陽衝撃+', 54=>'陰陽倶錯'},                              #  5月
	{   5=>'陰錯',     42=>'遂陣',      43=>'陽錯',     49=>'陽破陰衝+', 53=>'陰錯',  #  6月
	   54=>'遂陣+',    55=>'陽錯'},
	{  21=>'三陰',     40=>'陰錯',      56=>'陽錯'},                                  #  7月
	{  45=>'陰道衝陽', 51=>'陰錯',      57=>'陽錯'},                                  #  8月
	{   2=>'孤辰+',    10=>'陰位',      14=>'孤辰+',    26=>'行很+',     27=>'絶陽',  #  9月
	   28=>'絶陽',     29=>'絶陽',      30=>'絶陽',     31=>'絶陽',      32=>'絶陽',
	   33=>'絶陽',     34=>'單陽',      38=>'了戻+',    46=>'陽錯',
	   50=>'孤辰陰錯'},
	{   1=>'孤辰',      5=>'陰陽衝破',  13=>'孤辰',     24=>'歳博',      25=>'孤辰',  # 10月
	   34=>'絶陽',     35=>'絶陰',      37=>'行很',     48=>'歳博',      49=>'陰錯了戻',
	   53=>'陰陽交破', 59=>'陽錯'},
	{  24=>'陰陽倶錯', 42=>'陰陽衝撃',  48=>'陰陽倶錯'},                              # 11月
	{  24=>'遂陣',     43=>'陽破陰衝',  48=>'遂陣',     49=>'陽錯+',     59=>'陰錯'}  # 12月
      ],

      #
      # 吉事注 '+' は甘露日のみ表示
      #
      '吉事注' => [
          # ６番目の要素は『暦の大事典』pp.376-378にある「節月と日の干支で撰日する」吉事注
          # (“あさまきよし”～“むきまきよし”, ただし“かまぬり”を除く)

	[ # 正月
	  ['三寶吉+', nil,    '雑事吉', '加冠拜官出行療病入學起土竪柱上梁修宅門戸井吉', 				 nil			, nil					], #  0 甲子
	  ['三寶吉+', '神吉', nil,	'塞穴吉',									 '(くらたて) (井ほり)'	, 'みそつくり'				], #  1 乙丑
	  ['三寶吉',  nil,    nil,	'加冠拜官結婚納婦嫁娶出行吉',							 nil			, nil					], #  2 丙寅
	  ['三寶吉',  '神吉', '雑事吉', '嫁娶納婦納徴療病移徒壊垣破屋解除除服斬草吉',					 '(あさまき)'		, nil					], #  3 丁卯
	  [nil,       nil,    nil,	'経絡市買納財吉',								 'みそつくり'		, nil					], #  4 戊辰
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官冊授祠祀裁衣移徒出行市買納財吉',					 '入学' 		, nil					], #  5 己巳
	  ['三寶吉',  '神吉', nil,	'加冠拜官裁衣祠祀出行安床帳吉', 						 nil			, nil					], #  6 庚午
	  ['三寶吉',  nil,    nil,	'漁吉', 									 'いちたち'		, nil					], #  7 辛未
	  ['三寶吉+', '神吉', nil,	'療病解除壊垣破屋葬斬草除服吉', 						 nil			, nil					], #  8 壬申
	  ['三寶吉',  '神吉', nil,	'剃頭解除壊垣破屋葬斬草除服吉', 						 nil			, nil					], #  9 癸酉
	  ['三寶吉+', nil,    nil,	'入學吉',									 nil			, nil					], # 10 甲戌
	  [nil,       nil,    '雑事吉', '移徒修宅井竈壁市買納財剃頭捕魚吉',						 nil			, nil					], # 11 乙亥
	  ['三寶吉+', nil,    nil,	'加冠拜官冊授祠祀出行療病嫁娶移徒納婦結婚入學起土竪柱上梁修宅門戸井竈碓斬草吉',  nil			, nil					], # 12 丙子
	  ['三寶吉',  '神吉', nil,	'塞穴井竈碓斬草吉',								 'ものたち 木こり'	, nil					], # 13 丁丑
	  ['三寶吉',  nil,    nil,	'加冠拜官冊授移徒出行結婚納徴修宅屋門戸竪柱上梁吉',				 nil			, nil					], # 14 戊寅
	  ['三寶吉+', '神吉', nil,	'加冠拜官冊授嫁娶納婦納徴解除結婚移徒出行療病竪柱上梁修宅門戸吉',	       'みそさけつくり たねまき', nil					], # 15 己卯
	  [nil,       nil,    nil,	'裁衣市買納財吉',								 nil			, nil					], # 16 庚辰
	  ['三寶吉',  nil,    nil,	'祠祀拜官裁衣市買納財吉',							 nil			, nil					], # 17 辛巳
	  ['三寶吉',  '神吉', '雑事吉', '祠祀加冠拜官出行竪柱上柱蓋屋修造門戸斬草安床帳吉',				 '木こり'		, nil					], # 18 壬午
	  ['三寶吉',  nil,    nil,	'漁吉', 									 'たねまき'		, nil					], # 19 癸未
	  ['三寶吉+', '神吉', nil,	'解除療病壊垣破屋吉',								 nil			, nil					], # 20 甲申
	  ['三寶吉',  '神吉', nil,	'解除剃頭壊垣破屋吉',								 nil			, nil					], # 21 乙酉
	  ['三寶吉+', nil,    nil,	'入學起土祠戸種蒔吉',								 nil			, nil					], # 22 丙戌
	  ['三寶吉+', nil,    nil,	'結婚裁衣市買納財種蒔移徒吉',							 nil			, nil					], # 23 丁亥
	  ['三寶吉+', nil,    '雑事吉', '加冠祠祀移徒出行療病入學修井竈碓吉',						 '(けんふく) (入学)'	, 'かといて'				], # 24 戊子
	  ['三寶吉+', nil,    nil,	'塞穴吉',									 '(井ほり)'		, nil					], # 25 己丑
	  ['三寶吉',  nil,    nil,	'拜官結婚納婦出行吉',								 nil			, nil					], # 26 庚寅
	  [nil,       nil,    nil,	'此日不裁衣',									 nil			, 'たねまき みそつくり'			], # 27 辛卯
	  [nil,       nil,    nil,	'祀戸経絡裁衣市買納財吉',							 nil			, '木こり'				], # 28 壬辰
	  ['三寶吉',  nil,    '雑事吉', '加冠拜官謝土修宅門戸裁衣市買納財吉',						 nil			, '入学'				], # 29 癸巳
	  ['三寶吉',  '神吉', '雑事吉', '加冠拜官祠祀安床帳吉', 							 'かといて ふねのり'	, 'かといて'				], # 30 甲午
	  ['三寶吉+', nil,    nil,	'漁吉', 									 nil			, 'たねまき'				], # 31 乙未
	  ['三寶吉+', '神吉', nil,	'療病解除壊垣葬斬草除服吉',							 nil			, 'たねまき'				], # 32 丙申
	  ['三寶吉',  '神吉', nil,	'壊垣解除除服葬斬草吉', 							 nil			, nil					], # 33 丁酉
	  [nil,       nil,    nil,	'入學吉',									 nil			, nil					], # 34 戊戌
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官移徒納徴剃頭竪柱納財市買上梁修宅門戸吉',				 nil			, '入学'				], # 35 己亥
	  ['三寶吉',  '神吉', '雑事吉', '加冠拜官冊授嫁娶納婦結婚納徴修宅門移徒出行療病入學吉', 			 nil			, nil					], # 36 庚子
	  ['三寶吉',  '神吉', nil,	'塞穴吉',									 'ものたち'		, nil					], # 37 辛丑
	  ['三寶吉',  nil,    '雑事吉', '結婚移徒納財出行吉',								 'かといて'		, 'けんふく'				], # 38 壬寅
	  ['三寶吉',  '神吉', '雑事吉', '祠祀結婚納徴壊垣破屋療病斬草吉',						 'あさまき'		, 'わたまし けんふく'			], # 39 癸卯
	  [nil,       nil,    nil,	'結婚市買納財吉',								 'かといて'		, nil					], # 40 甲辰
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官冊授市買納財吉',							 'けんふく'		, nil					], # 41 乙巳
	  ['三寶吉',  '神吉', '雑事吉', '拜官祠祀移徒謝土修造門戸上梁葬斬草安床帳吉',					 'わたまし'		, 'わたまし いちたち 木こり'		], # 42 丙午
	  ['三寶吉',  '神吉', nil,	'漁吉', 									 'かといて'		, 'あさまき 木こり'			], # 43 丁未
	  ['三寶吉',  '神吉', nil,	'壊垣破屋療病解除除服吉',							 'たねまき'		, 'たねまき'				], # 44 戊申
	  ['三寶吉',  '神吉', nil,	'移徒剃頭解除壊垣除服吉',							 nil			, nil					], # 45 己酉
	  [nil,       nil,    nil,	'此日不見死人', 								 nil			, nil					], # 46 庚戌
	  ['三寶吉',  '神吉', nil,	'加冠拜官冊授祠祀納徴市買納財起土吉',						 nil			, nil					], # 47 辛亥
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官結婚納徴出行移徒療病入學起土修宅門戸井碓磑種蒔斬草吉', 		 nil			, 'いちたち'				], # 48 壬子
	  ['三寶吉',  '神吉', nil,	'塞穴吉',									 nil			, nil					], # 49 癸丑
	  [nil,       nil,    nil,	'此日為解除不行病人家', 							 nil			, nil					], # 50 甲寅
	  ['三寶吉',  '神吉', '雑事吉', '結婚納徴移徒解除療病壊垣破屋斬草吉',						 'ものたち'		, 'けんふく'				], # 51 乙卯
	  [nil,       nil,    nil,	'結婚裁衣市買納財吉',								 'すゝはらひ'		, nil					], # 52 丙辰
	  ['三寶吉',  nil,    '雑事吉', '加冠拜官祠祀移徒出行市買納財吉',						 nil			, nil					], # 53 丁巳
	  ['三寶吉+', '神吉', '雑事吉', '加冠祠祀市買納財出行上梁安床帳吉',					       'ふねのり 正月ことはしめ', nil					], # 54 戊午
	  ['三寶吉',  '神吉', nil,	'漁吉', 							      '(かといて) (たねまき) すゝはらひ', 'かといて たねまき'			], # 55 己未
	  [nil,       '神吉', nil,	'療病解除壊垣破屋吉',								 nil			, nil					], # 56 庚申
	  ['三寶吉',  '神吉', nil,	'解除剃頭壊垣破屋除服葬斬草吉', 						 nil			, nil					], # 57 辛酉
	  ['三寶吉+', nil,    nil,	'入學吉',									 nil			, nil					], # 58 壬戌
	  ['三寶吉+', nil,    nil,	'剃頭謝土市買納財修井竈碓吉',							 '(けんふく)'		, nil					], # 59 癸亥
	],
	[ # 2月
	  ['三寶吉+', nil,    '雑事吉', '祠祀出行移徒上梁修理門戸井竈碓磑種蒔吉',					 nil			, 'たねかし'				], #  0 甲子
	  [nil,       nil,    nil,	'入學療病市買納財吉',								 'ものたち'		, 'すつくり'				], #  1 乙丑
	  ['三寶吉',  nil,    nil,	'塞穴吉',									 nil			, nil					], #  2 丙寅
	  ['三寶吉',  nil,    nil,	'結婚出行吉',									 nil			, nil					], #  3 丁卯
	  ['三寶吉+', nil,    nil,	'療病解除元服壊垣吉',								 nil			, nil					], #  4 戊辰
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官冊授祠祀裁衣経絡市買納財奴婢六蓄安床帳吉',				 nil			, nil					], #  5 己巳
	  ['三寶吉',  '神吉', nil,	'加冠拜官謝土市買納財吉',							 nil			, nil					], #  6 庚午
	  ['三寶吉',  nil,    nil,	'裁衣市買納財安床帳吉', 							 'くらたて 井ほり'	, nil					], #  7 辛未
	  ['三寶吉+', '神吉', nil,	'解除漁葬斬草除服吉',								 '入学'			, '入学'				], #  8 壬申
	  ['三寶吉',  nil,    nil,	'療病剃頭壊垣破屋解除元服葬斬草吉',						 'やたて'		, 'たねかし たねまき'			], #  9 癸酉
	  ['三寶吉+', nil,    '雑事吉', '壊垣破屋吉',									 'かといて'		, 'わたまし'				], # 10 甲戌
	  [nil,       nil,    '雑事吉', '祠祀移徒入學剃頭起土竪柱上梁修宅門戸井竈碓磑裁衣市買納財吉',			 'みそすつくり' 	, 'たねかし みそすつくり'		], # 11 乙亥
	  ['三寶吉+', nil,    nil,	'加冠嫁娶竪柱納婦結婚移徒起土上梁蓋屋修宅門戸種蒔斬草吉',			 'たねかし'		, 'あさまき たねかし'			], # 12 丙子
	  [nil,       nil,    nil,	'拜官結婚嫁娶療病入學吉',							 'ものたち'		, nil					], # 13 丁丑
	  ['三寶吉',  nil,    nil,	'加冠拜官冊授竪柱上梁蓋屋修宅井門戸竈碓謝土塞穴吉',				 nil			, nil					], # 14 戊寅
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 15 己卯
	  ['三寶吉',  nil,    '雑事吉', '療病解除壊垣破屋吉',								 nil			, nil					], # 16 庚辰
	  ['三寶吉',  nil,    nil,	'加冠拜官冊授出行裁衣経絡市買納財安床帳吉',					 nil			, nil					], # 17 辛巳
	  ['三寶吉',  '神吉', '雑事吉', '加冠祠祀移徒起土竪柱上梁蓋屋修宅門戸吉',					 nil			, 'けんふく'				], # 18 壬午
	  ['三寶吉',  nil,    nil,	'裁衣市買納財安床帳吉', 							 nil			, 'たねかし'				], # 19 癸未
	  ['三寶吉+', '神吉', nil,	'漁解除除服元服葬斬草吉',							 '入学' 		, '入学'				], # 20 甲申
	  ['三寶吉',  nil,    nil,	'壊垣破屋療病解除剃頭吉',							 nil			, 'たねまき'				], # 21 乙酉
	  ['三寶吉+', nil,    nil,	'壊垣破屋吉',									 'あさまき'		, 'たねまき'				], # 22 丙戌
	  ['三寶吉+', nil,    nil,	'結婚嫁娶納婦移徒入學裁衣竪柱上梁修門戸井碓吉', 				 nil			, nil					], # 23 丁亥
	  ['三寶吉+', nil,    '雑事吉', '祠祀移徒出行竪柱上梁修門戸井碓磑謝土種蒔吉',					 'けんふく たねかし'	, nil					], # 24 戊子
	  [nil,       nil,    nil,	'療病吉',									 '井ほり'		, 'いちたち'				], # 25 己丑
	  ['三寶吉',  nil,    nil,	'塞穴殯埋斬草吉',								 nil			, nil					], # 26 庚寅
	  ['三寶吉+', nil,    nil,	'出行吉',									 nil			, nil					], # 27 辛卯
	  ['三寶吉+', nil,    nil,	'療病解除元服壊垣破屋吉',							 nil			, nil					], # 28 壬辰
	  ['三寶吉',  nil,    '雑事吉', '加冠拜官冊授出行移徒裁衣竪柱納財上梁修宅門戸経絡市買安床帳吉', 		 nil			, nil					], # 29 癸巳
	  ['三寶吉',  '神吉', '雑事吉', '拜官移徒出行上梁修宅門戸井竈碓磑殯理斬草吉',					 nil			, nil					], # 30 甲午
	  ['三寶吉+', nil,    nil,	'裁衣市買納財葬斬草安床帳吉',							 'ふねのり'		, nil					], # 31 乙未
	  ['三寶吉+', '神吉', nil,	'解除除服元服葬斬草吉', 							 nil			, nil					], # 32 丙申
	  ['三寶吉',  nil,    nil,	'療病入學解除元服壊垣殯理斬草吉',						 'あさまき たねかし'	, nil					], # 33 丁酉
	  [nil,       nil,    nil,	'壊垣破屋吉',									 'かといて'		, nil					], # 34 戊戌
	  ['三寶吉+', '神吉', '雑事吉', '結婚納徴嫁娶市買納財剃頭移徒入學上梁修宅吉',					 'みそすつくり' 	, nil					], # 35 己亥
	  ['三寶吉',  '神吉', '雑事吉', '加冠祠祀結婚嫁娶移徒出行上梁蓋屋種蒔吉',					 'よめとり たねかし'	, nil					], # 36 庚子
	  [nil,       nil,    nil,	'療病入學市買納財吉',								 'ものたち'		, nil					], # 37 辛丑
	  ['三寶吉',  nil,    nil,	'塞穴葬斬草吉', 								 'やたて くらたて'	, nil					], # 38 壬寅
	  ['三寶吉',  nil,    nil,	'出行吉',									 nil			, nil					], # 39 癸卯
	  ['三寶吉',  nil,    '雑事吉', '療病解除壊垣吉',								 nil			, nil					], # 40 甲辰
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官冊授裁衣市買納財安床帳吉',						 nil			, nil					], # 41 乙巳
	  ['三寶吉',  '神吉', '雑事吉', '加冠拜官祠祀謝土吉',								 nil			, nil					], # 42 丙午
	  ['三寶吉',  '神吉', nil,	'入學裁衣市買納財安床帳吉',							 nil			, nil					], # 43 丁未
	  ['三寶吉',  '神吉', nil,	'解除元服漁吉', 								 'たねまき'		, nil					], # 44 戊申
	  ['三寶吉',  nil,    nil,	'壊垣破屋療病解除元服剃頭除服葬斬草吉', 					 nil			, nil					], # 45 己酉
	  ['三寶吉+', nil,    nil,	'壊垣破屋吉',									 'けんふく'		, nil					], # 46 庚戌
	  ['三寶吉',  '神吉', nil,	'加冠拜官冊授移徒入學剃頭市買納財婢六蓄起土竪柱吉',				 nil			, nil					], # 47 辛亥
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官冊授祠祀結婚納徴起土上梁修宅門戸井竈碓謝土吉', 			 'あさまき'		, nil					], # 48 壬子
	  [nil,       nil,    nil,	'療病入學市買納財吉',								 nil			, nil					], # 49 癸丑
	  ['三寶吉',  nil,    nil,	'加冠拜官塞穴殯埋斬草吉',							 'やたて'		, nil					], # 50 甲寅
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 51 乙卯
	  ['三寶吉+', nil,    '雑事吉', '療病解除壊垣破屋吉',								 nil			, nil					], # 52 丙辰
	  ['三寶吉',  nil,    '雑事吉', '加冠拜官裁衣経絡謝土市買納財安床帳吉', 					 nil			, nil					], # 53 丁巳
	  ['三寶吉+', '神吉', '雑事吉', '加冠出行移徒修井竈碓吉',							 nil			, nil					], # 54 戊午
	  ['三寶吉',  '神吉', nil,	'裁衣市買納財安床帳吉', 							 'たねかし'		, 'いちたち かといて'			], # 55 己未
	  [nil,       '神吉', nil,	'漁解除元服葬斬草吉',								 nil			, nil					], # 56 庚申
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 57 辛酉
	  ['三寶吉+', nil,    nil,	'壊垣破屋吉',									 'あさまき'		, 'あさまき たねかし'			], # 58 壬戌
	  ['三寶吉+', nil,    nil,	'祠祀剃頭入學裁衣市買納財吉',							 'たねかし'		, '入学 たねかし'			], # 59 癸亥
	],
	[ # 3月
	  [nil,       nil,    nil,	nil,										 nil			, nil					], #  0 甲子
	  [nil,       nil,    nil,	nil,										 nil			, nil					], #  1 乙丑
	  [nil,       nil,    nil,	nil,										 nil			, nil					], #  2 丙寅
	  [nil,       nil,    nil,	nil,										 'たねまき'		, nil					], #  3 丁卯
	  [nil,       nil,    nil,	nil,										 nil			, nil					], #  4 戊辰
	  ['三寶吉+', '神吉', '雑事吉', '拜官冊授裁衣療病解除壊垣破屋市買納財吉',					 'かといて'		, nil					], #  5 己巳
	  ['三寶吉',  '神吉', nil,	'剃頭修堤防導達溝渠祀戸葬斬草安床帳吉', 					 'みそすつくり' 	, nil					], #  6 庚午
	  ['三寶吉',  nil,    nil,	'裁衣市買納財吉',								 nil			, nil					], #  7 辛未
	  [nil,       nil,    nil,	nil,										 nil			, nil					], #  8 壬申
	  ['三寶吉',  '神吉', nil,	'祠祀解除除服元服葬斬草吉',							 nil			, nil					], #  9 癸酉
	  [nil,       nil,    nil,	'療病吉',									 'くさかり'		, nil					], # 10 甲戌
	  [nil,       nil,    '雑事吉', '結婚納徴納財裁衣壊垣破屋市買納婦吉',						 nil			, nil					], # 11 乙亥
	  ['三寶吉+', nil,    nil,	'入學拜官修理井竈殯埋斬草吉',							 nil			, nil					], # 12 丙子
	  ['三寶吉',  '神吉', nil,	'結婚納徴嫁娶納婦吉',								 nil			, nil					], # 13 丁丑
	  ['三寶吉',  nil,    nil,	'出行療病修井竈碓導達溝渠埋堤防吉',						 nil			, nil					], # 14 戊寅
	  ['三寶吉+', '神吉', nil,	'拜官祠祀結婚納徴療病剃頭謝土宅堤防塞穴吉',					 nil			, nil					], # 15 己卯
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 16 庚辰
	  ['三寶吉',  nil,    nil,	'拜官療病解除拂舎起土修井冊授竈碓市買納財吉',					 nil			, nil					], # 17 辛巳
	  ['三寶吉',  '神吉', '雑事吉', '祠祀剃頭経絡謝土修井碓導達溝渠修理堤防葬斬草吉',				 nil			, nil					], # 18 壬午
	  ['三寶吉',  nil,    nil,	'裁衣市買納財吉',								 nil			, nil					], # 19 癸未
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 20 甲申
	  ['三寶吉',  '神吉', nil,	'解除除服元服吉',								 'くさかり'		, nil					], # 21 乙酉
	  [nil,       nil,    nil,	'療病壊垣破屋吉',								 nil			, nil					], # 22 丙戌
	  ['三寶吉+', nil,    nil,	'裁衣市買納財吉',								 nil			, nil					], # 23 丁亥
	  ['三寶吉+', nil,    nil,	'拜官祠祀導達溝渠吉',								 nil			, nil					], # 24 戊子
	  ['三寶吉+', nil,    nil,	'嫁娶吉',									 nil			, nil					], # 25 己丑
	  ['三寶吉',  nil,    nil,	'結婚納徴出行修理療病殯理斬草吉',						 'いちたち'		, nil					], # 26 庚寅
	  ['三寶吉+', '神吉', '雑事吉', '拜官結婚納徴剃頭修堤防雍水塞穴斬草吉', 					 'わたまし ものたち'	, 'わたまし たねまき'			], # 27 辛卯
	  ['三寶吉+', nil,    nil,	'出行吉',									 nil			, 'いちたち'				], # 28 壬辰
	  ['三寶吉',  nil,    '雑事吉', '拜官冊授療病解除市買納財吉',							 'けんふく'		, 'けんふく'				], # 29 癸巳
	  ['三寶吉',  '神吉', '雑事吉', '拜官祠祀出行経絡剃頭修井竈導達溝渠斬草安床帳吉',				 'ふねのり'		, 'かといて'				], # 30 甲午
	  ['三寶吉+', nil,    nil,	'裁衣市買納財吉',								 'ものたち'		, nil					], # 31 乙未
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 32 丙申
	  ['三寶吉',  '神吉', nil,	'解除謝土除服殯埋吉',								 'いちたち'		, 'いちたち'				], # 33 丁酉
	  [nil,       nil,    nil,	'療病壊垣吉',									 nil			, nil					], # 34 戊戌
	  ['三寶吉+', '神吉', '雑事吉', '嫁娶納徴裁衣市買納財吉',							 nil			, nil					], # 35 己亥
	  ['三寶吉',  '神吉', nil,	'祠祀謝土修理上梁導達溝渠斬草吉',						 nil			, nil					], # 36 庚子
	  ['三寶吉',  '神吉', nil,	'市買納財種蒔吉',								 nil			, nil					], # 37 辛丑
	  ['三寶吉',  nil,    nil,	'結婚納徴出行上梁療病修堤防導達溝渠葬斬草吉',					 nil			, nil					], # 38 壬寅
	  ['三寶吉',  '神吉', '雑事吉', '剃頭塞穴殯埋斬草吉',								 'けんふく かといて'	, 'わたまし けんふく かといて たねまき'	], # 39 癸卯
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 40 甲辰
	  ['三寶吉+', '神吉', '雑事吉', '拜官冊授裁衣療病市買納財解除壊垣吉',						 'ふねのり'		, nil					], # 41 乙巳
	  ['三寶吉',  '神吉', '雑事吉', '出行経絡剃頭謝土修井堤防導達溝渠葬斬草安床帳吉',				 nil			, nil					], # 42 丙午
	  ['三寶吉',  '神吉', nil,	'裁衣市買納財吉',								 nil			, 'いちたち'				], # 43 丁未
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 44 戊申
	  ['三寶吉',  '神吉', nil,	'祠祀納徴解除修井碓磑導達溝渠吉',						 nil			, nil					], # 45 己酉
	  [nil,       nil,    nil,	'療病壊垣破屋吉',								 nil			, nil					], # 46 庚戌
	  ['三寶吉',  '神吉', nil,	'裁衣市買納財壊垣破屋吉',							 nil			, nil					], # 47 辛亥
	  ['三寶吉+', '神吉', nil,	'祠祀修井竈碓殯埋堤防導達溝渠謝土種蒔吉',					 nil			, 'くさかり'				], # 48 壬子
	  ['三寶吉',  '神吉', nil,	nil,										 nil			, nil					], # 49 癸丑
	  ['三寶吉',  nil,    nil,	'療病出行修堤防導達溝渠斬草吉', 						 'やたて'		, nil					], # 50 甲寅
	  ['三寶吉',  '神吉', '雑事吉', '結婚納徴剃頭塞穴殯埋斬草吉',							 'ものたち'		, 'けんふく'				], # 51 乙卯
	  ['三寶吉+', nil,    '雑事吉', '出行吉',									 'わたまし'		, nil					], # 52 丙辰
	  ['三寶吉',  nil,    '雑事吉', '拜官冊授療病解除掃舎裁衣市買納財壊垣破屋吉',					 'いちたち'		, 'たねまき'				], # 53 丁巳
	  ['三寶吉+', '神吉', '雑事吉', '拜官祠祀経絡剃頭修堤防導達溝渠安床帳吉',					 'ふねのり'		, nil					], # 54 戊午
	  ['三寶吉',  '神吉', nil,	'裁衣市買納財吉',								 nil			, nil					], # 55 己未
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 56 庚申
	  ['三寶吉',  '神吉', nil,	'解除除服元服葬斬草吉', 							 '井ほり'		, nil					], # 57 辛酉
	  [nil,       nil,    nil,	'療病壊垣破屋吉',								 nil			, nil					], # 58 壬戌
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 59 癸亥
	],
	[ # 4月
	  ['三寶吉+', nil,    '雑事吉', '祠祀嫁娶納徴移徒壊垣破屋謝土吉',						 nil			, 'よめとり さひらき'			], #  0 甲子
	  ['三寶吉+', nil,    nil,	'加冠拜官冊授祠祀入學修井碓吉', 						 'かまぬり'		, nil					], #  1 乙丑
	  ['三寶吉',  nil,    nil,	'加冠拜官冊授結婚納徴納婦移徒出行修理種蒔吉',					 nil			, nil					], #  2 丙寅
	  ['三寶吉',  '神吉', '雑事吉', '加冠拜官冊授結婚出行祠祀入學移徒納婦殯理斬草吉',				 'わたまし'		, 'わたまし 入学'			], #  3 丁卯
	  [nil,       nil,    nil,	nil,										 nil			, nil					], #  4 戊辰
	  [nil,       nil,    nil,	nil,										 nil			, nil					], #  5 己巳
	  ['三寶吉',  '神吉', nil,	'拜官祠祀移徒剃頭療病解除除服修竈碓磑斬草吉',					 'さひらき みそすつくり', 'みそすつくり'			], #  6 庚午
	  [nil,       nil,    nil,	nil,										 nil			, nil					], #  7 辛未
	  ['三寶吉+', '神吉', nil,	'移徒解除市買納財吉',								 nil			, '入学'				], #  8 壬申
	  ['三寶吉',  '神吉', nil,	'裁衣剃頭解除謝土祠祀修竈葬斬草除服安床帳吉',					 nil			, nil					], #  9 癸酉
	  ['三寶吉+', nil,    '雑事吉', '結婚納徴嫁娶吉',								 'やたて くらたて'	, nil					], # 10 甲戌
	  [nil,       nil,    '雑事吉', '療病裁衣市買納財壊垣破屋吉',							 nil			, 'けんふく'				], # 11 乙亥
	  ['三寶吉+', nil,    nil,	'嫁娶納徴壊垣破屋吉',								 'さひらき'		, 'さひらき たねまき'			], # 12 丙子
	  ['三寶吉',  nil,    nil,	'入學吉',									 'ものたち'		, nil					], # 13 丁丑
	  ['三寶吉',  nil,    nil,	'加冠拜官移徒修竈井種蒔吉',							 'たねまき'		, nil					], # 14 戊寅
	  ['三寶吉+', '神吉', nil,	'加冠拜官冊授祠祀結婚納婦納徴移徒出行療病入學謝土吉',				 '入学' 		, '入学'				], # 15 己卯
	  ['三寶吉',  nil,    '雑事吉', '雍水塞穴吉',									 'けんふく'		, 'けんふく いちたち'			], # 16 庚辰
	  ['三寶吉',  nil,    nil,	'加冠拜官冊授移徒出行裁衣市買納財吉',						 nil			, nil					], # 17 辛巳
	  ['三寶吉',  '神吉', '雑事吉', '拜官移徒剃頭療病壊垣破屋解除吉',						 nil			, 'かといて'				], # 18 壬午
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 19 癸未
	  ['三寶吉+', '神吉', nil,	'結婚納徴移徒解除除服謝土吉',							 nil			, nil					], # 20 甲申
	  ['三寶吉',  '神吉', nil,	'嫁娶納婦移徒出行剃頭解除除服修井竈磑安床帳吉', 				 nil			, nil					], # 21 乙酉
	  ['三寶吉+', nil,    nil,	'結婚納徴嫁娶納婦吉',								 'くさかり'		, 'さひらき くさかり'			], # 22 丙戌
	  ['三寶吉+', nil,    nil,	'療病裁衣市買納財壊垣破屋吉',							 nil			, nil					], # 23 丁亥
	  ['三寶吉+', nil,    '雑事吉', '結婚納徴嫁娶納婦壊垣吉',							 'けんふく'		, 'わたまし けんふく くさかり'		], # 24 戊子
	  ['三寶吉+', nil,    nil,	'入學吉',									 nil			, nil					], # 25 己丑
	  ['三寶吉',  nil,    nil,	'拜官結婚納徴種蒔移徒謝土修井竈碓殯埋斬草吉',					 'さひらき'		, nil					], # 26 庚寅
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官冊授祠祀出行結婚療病入學修碓斬草吉',					 'やたて ものたち'	, 'さひらき'				], # 27 辛卯
	  ['三寶吉+', nil,    nil,	'塞穴吉',									 nil			, nil					], # 28 壬辰
	  ['三寶吉',  nil,    '雑事吉', '拜官裁衣出行市買納財祠祀竈吉', 						 nil			, nil					], # 29 癸巳
	  ['三寶吉',  '神吉', '雑事吉', '加冠拜官冊授祠祀療病剃頭解除修井竈碓謝土斬草除服吉',				 nil			, 'くさかり'				], # 30 甲午
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 31 乙未
	  ['三寶吉+', '神吉', nil,	'解除移徒修井磑吉',								 '入学' 		, nil					], # 32 丙申
	  ['三寶吉',  '神吉', nil,	'結婚嫁娶納婦解除元服葬斬草安床帳吉',						 nil			, nil					], # 33 丁酉
	  [nil,       nil,    nil,	'嫁娶吉',									 'くらたて かまぬり'	, nil					], # 34 戊戌
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 35 己亥
	  ['三寶吉',  '神吉', '雑事吉', '祠祀壊垣破屋斬草吉',								 'わたまし たねまき'	, nil					], # 36 庚子
	  ['三寶吉',  nil,    nil,	'入學吉',									 nil			, nil					], # 37 辛丑
	  ['三寶吉',  nil,    '雑事吉', '拜官納徴結婚修井碓種蒔吉',							 nil			, nil					], # 38 壬寅
	  ['三寶吉',  '神吉', '雑事吉', '加冠祠祀結婚納徴療病入學移徒出行祀竈斬草吉',					 'かといて さひらき'	, nil					], # 39 癸卯
	  ['三寶吉',  nil,    '雑事吉', '塞穴吉',									 nil			, nil					], # 40 甲辰
	  ['三寶吉+', '神吉', '雑事吉', '出行裁衣市買納財吉',								 nil			, nil					], # 41 乙巳
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 42 丙午
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 43 丁未
	  ['三寶吉',  '神吉', nil,	'解除除服吉',									 nil			, nil					], # 44 戊申
	  ['三寶吉',  '神吉', nil,	'出行移徒剃頭謝土修井竈碓葬解除元服安床帳裁衣吉',				 nil			, nil					], # 45 己酉
	  ['三寶吉+', nil,    nil,	'移徒吉',									 'やたて くさかり'	, nil					], # 46 庚戌
	  ['三寶吉',  '神吉', nil,	'療病裁衣納財壊垣破屋吉',							 'さひらき'		, nil					], # 47 辛亥
	  ['三寶吉+', '神吉', '雑事吉', '移徒修井竈壊垣破屋吉', 							 'くらたて'		, nil					], # 48 壬子
	  ['三寶吉',  nil,    nil,	'入學謝土祀竈吉',								 nil			, nil					], # 49 癸丑
	  ['三寶吉',  nil,    nil,	'拜官結婚納徴移徒上梁修井竈碓種蒔斬草吉',					 nil			, nil					], # 50 甲寅
	  ['三寶吉',  '神吉', '雑事吉', '加冠祠祀結婚納徴納婦移徒出行療病入學修竈磑謝土殯理斬草吉',			 'さひらき'		, nil					], # 51 乙卯
	  ['三寶吉+', nil,    '雑事吉', '塞穴吉',									 nil			, nil					], # 52 丙辰
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 53 丁巳
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 54 戊午
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 55 己未
	  [nil,       '神吉', nil,	'祠祀解除元服吉',								 nil			, nil					], # 56 庚申
	  ['三寶吉',  '神吉', nil,	'移徒剃頭謝土葬斬草除服安床帳解除吉',						 nil			, nil					], # 57 辛酉
	  ['三寶吉+', nil,    nil,	'裁衣吉',									 nil			, nil					], # 58 壬戌
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 59 癸亥
	],
	[ # 5月
	  ['三寶吉+', nil,    nil,	'療病修宅壊垣破屋吉',								 nil			, nil					], #  0 甲子
	  ['三寶吉+', '神吉', nil,	'壊垣破屋吉',									 nil			, nil					], #  1 乙丑
	  ['三寶吉',  nil,    nil,	'加冠拜官冊授結婚入學謝土殯埋納婦修宅移徒吉',					 nil			, nil					], #  2 丙寅
	  [nil,       nil,    nil,	'加冠拜官祠祀結婚修竈碓移徒謝土納婦吉', 					 nil			, nil					], #  3 丁卯
	  ['三寶吉+', nil,    nil,	'出行入學療病種蒔吉',								 'かまぬり 田うへ'	, nil					], #  4 戊辰
	  ['三寶吉+', '神吉', '雑事吉', '拜官市買納財塞穴裁衣吉',							 nil			, nil					], #  5 己巳
	  ['三寶吉',  nil,    nil,	'剃頭吉',									 'むきかり'		, nil					], #  6 庚午
	  ['三寶吉',  nil,    nil,	'解除壊垣破屋吉',								 'ふねのり 井ほり'	, nil					], #  7 辛未
	  ['三寶吉+', '神吉', nil,	'祠祀移徒経絡解除除服元服葬斬草吉',						 'くさかり'		, nil					], #  8 壬申
	  ['三寶吉',  '神吉', nil,	'祠祀嫁娶納婦剃頭解除謝土祀竈吉',						 nil			, nil					], #  9 癸酉
	  ['三寶吉+', nil,    '雑事吉', '祠祀嫁娶納婦安床帳吉', 							 'わたまし くらたて'	, nil					], # 10 甲戌
	  [nil,       nil,    '雑事吉', '嫁娶裁衣移徒市買納財吉',							 'むきかり 田うへ'	, nil					], # 11 乙亥
	  ['三寶吉+', nil,    nil,	'拜官療病壊垣破屋斬草吉',							 nil			, nil					], # 12 丙子
	  ['三寶吉',  '神吉', nil,	'裁衣市買納財壊垣破屋吉',							 'ものたち'		, nil					], # 13 丁丑
	  ['三寶吉',  nil,    nil,	'加冠拜官結婚納徴入學修磑吉',							 'みそすつくり' 	, nil					], # 14 戊寅
	  [nil,       nil,    nil,	'祠祀結婚納徴納婦移徒謝土吉',							 nil			, nil					], # 15 己卯
	  ['三寶吉',  nil,    '雑事吉', '出行入學療病修竈種蒔吉',							 'けんふく かといて'	, nil					], # 16 庚辰
	  ['三寶吉',  nil,    nil,	'拜官冊授裁衣市買納財塞穴吉',							 nil			, nil					], # 17 辛巳
	  ['三寶吉',  nil,    nil,	'加冠拜官吉',									 nil			, nil					], # 18 壬午
	  ['三寶吉',  nil,    nil,	'結婚納徴壊垣破屋解除祀竈吉',							 '田うへ'		, nil					], # 19 癸未
	  ['三寶吉+', '神吉', nil,	'祠祀嫁娶解除謝土葬斬草吉',							 nil			, nil					], # 20 甲申
	  ['三寶吉',  '神吉', nil,	'嫁娶納婦移徒剃頭解除吉',							 nil			, nil					], # 21 乙酉
	  ['三寶吉+', nil,    nil,	'結婚納徴嫁娶納婦安床帳吉',							 'むきかり'		, nil					], # 22 丙戌
	  ['三寶吉+', nil,    nil,	'裁衣市買納財吉',								 nil			, nil					], # 23 丁亥
	  ['三寶吉+', nil,    nil,	'療病壊垣吉',									 nil			, nil					], # 24 戊子
	  ['三寶吉+', nil,    nil,	'壊垣吉',									 '田うへ'		, 'むきかり'				], # 25 己丑
	  ['三寶吉',  nil,    nil,	'加冠拜官入學謝土修磑斬草吉',							 '井ほり'		, nil					], # 26 庚寅
	  [nil,       nil,    nil,	'加冠拜官冊授祠祀結婚納徴移徒吉',						 'ものたち'		, nil					], # 27 辛卯
	  ['三寶吉+', nil,    nil,	'出行入學療病種蒔吉',								 nil			, nil					], # 28 壬辰
	  ['三寶吉',  nil,    '雑事吉', '拜官裁衣市買納財塞穴祀竈吉',							 nil			, nil					], # 29 癸巳
	  ['三寶吉',  nil,    nil,	'加冠拜官冊授移徒裁衣修竈磑吉', 						 nil			, nil					], # 30 甲午
	  ['三寶吉+', nil,    nil,	'解除壊垣破屋吉',								 'むきかり'		, 'むきかり'				], # 31 乙未
	  ['三寶吉+', '神吉', nil,	'祠祀嫁娶納婦移徒経絡謝土解除元服葬斬草吉',					 '入学 いちたち'	,  '入学 いちたち たねまき'		], # 32 丙申
	  ['三寶吉',  '神吉', nil,	'祠祀解除吉',									 nil			, nil					], # 33 丁酉
	  [nil,       nil,    nil,	'嫁娶安床帳吉', 								 'かまぬり 田うへ'	, nil					], # 34 戊戌
	  ['三寶吉+', '神吉', '雑事吉', '裁衣市買納財吉',								 nil			, nil					], # 35 己亥
	  ['三寶吉',  nil,    nil,	'療病壊垣斬草吉',								 nil			, nil					], # 36 庚子
	  ['三寶吉',  '神吉', nil,	'壊垣吉',									 nil			, nil					], # 37 辛丑
	  ['三寶吉',  nil,    nil,	'拜官入學修磑結婚吉',								 'くさかり'		, nil					], # 38 壬寅
	  [nil,       nil,    nil,	'祠祀結婚納徴移徒謝土祀竈吉',							 nil			, 'むきかり'				], # 39 癸卯
	  ['三寶吉',  nil,    '雑事吉', '出行入學療病種蒔吉',								 'むきかり 田うへ'	, 'たねまき'				], # 40 甲辰
	  ['三寶吉+', '神吉', '雑事吉', '裁衣市買納財塞穴吉',								 nil			, nil					], # 41 乙巳
	  [nil,       nil,    nil,	'此日不誓願',									 nil			, nil					], # 42 丙午
	  ['三寶吉',  '神吉', nil,	'裁衣解除壊垣吉',								 nil			, 'かといて'				], # 43 丁未
	  ['三寶吉',  '神吉', nil,	'嫁娶納徴納婦経絡解除除服吉',							 '入学' 		, 'くさかり むきかり'			], # 44 戊申
	  ['三寶吉',  '神吉', nil,	'祠祀剃頭移徒出行修竈磑謝土解除除服吉', 					 nil			, nil					], # 45 己酉
	  ['三寶吉+', nil,    nil,	'移徒謝土市買納財安床帳吉',							 'かといて'		, 'かといて'				], # 46 庚戌
	  ['三寶吉',  '神吉', nil,	'移徒裁衣造作竈碓市買納財吉',							 '入学' 		, nil					], # 47 辛亥
	  [nil,       nil,    nil,	'此日不打人又不動財',								 nil			, nil					], # 48 壬子
	  ['三寶吉',  '神吉', nil,	'裁衣祀竈壊垣破屋吉',								 'むきかり'		, 'むきかり'				], # 49 癸丑
	  ['三寶吉',  nil,    nil,	'加冠拜官入學修碓殯埋斬草吉',							 '田うへ'		, nil					], # 50 甲寅
	  [nil,       nil,    nil,	'拜官祠祀結婚納徴移徒謝土吉',							 'ものたち'		, nil					], # 51 乙卯
	  ['三寶吉+', nil,    '雑事吉', '出行入學療病種蒔壊垣吉',							 'けんふく いちたち'	, 'けんふく いちたち むきかり'		], # 52 丙辰
	  ['三寶吉',  nil,    '雑事吉', '拜官裁衣市買納財塞穴吉',							 nil			, nil					], # 53 丁巳
	  [nil,       nil,    nil,	'此日不誓道乃神',								 nil			, nil					], # 54 戊午
	  ['三寶吉',  '神吉', nil,	'解除壊垣吉',									 'みそすつくり 田うへ'	, 'みそすつくり むきかり'		], # 55 己未
	  [nil,       '神吉', nil,	'出行移徒解除葬斬草除服吉',							 '入学 むきかり'	, nil					], # 56 庚申
	  ['三寶吉',  '神吉', nil,	'祠祀移徒剃頭解除除服吉',							 nil			, nil					], # 57 辛酉
	  ['三寶吉+', nil,    nil,	'裁衣市買納財安床帳吉', 							 'むきかり'		, 'たねまき'				], # 58 壬戌
	  ['三寶吉+', nil,    nil,	'結婚嫁娶納婦市買納財祀竈吉',							 nil			, nil					], # 59 癸亥
	],
	[ # 6月
	  [nil,       nil,    nil,	'祠祀作竈磑吉', 								 nil			, nil					], #  0 甲子
	  ['三寶吉+', '神吉', nil,	'療病壊垣破屋吉',								 nil			, nil					], #  1 乙丑
	  ['三寶吉',  nil,    nil,	'拜官結婚納徴斬草吉',								 'わたまし'		, 'わたまし'				], #  2 丙寅
	  ['三寶吉',  '神吉', '雑事吉', '拜官祠祀結婚納徴殯埋斬草吉',							 nil			, nil					], #  3 丁卯
	  ['三寶吉+', nil,    nil,	'市買納財種蒔吉',								 nil			, nil					], #  4 戊辰
	  [nil,       nil,    nil,	'此日不見死人日乃可加乃神也',							 nil			, nil					], #  5 己巳
	  ['三寶吉',  '神吉', nil,	'剃頭塞穴斬草吉',								 nil			, nil					], #  6 庚午
	  ['三寶吉',  nil,    nil,	'出行吉或又云此日節中出行定死又不可還未云々',					 'かといて ふねのり'	, 'かといて'				], #  7 辛未
	  ['三寶吉+', '神吉', nil,	'嫁娶療病解除壊垣破屋掃舎葬斬草除服吉此日安床帳吉此日安床帳不立在鬼仍不可用',	 'くさかり'		, 'くさかり'				], #  8 壬申
	  ['三寶吉',  '神吉', nil,	'嫁娶経絡謝土祀竈解除葬斬草除服安床帳吉此日安床帳不魂仍不可用', 		 'みそすつくり' 	, 'みそすつくり'			], #  9 癸酉
	  ['三寶吉+', nil,    '雑事吉', '嫁娶吉',									 'わたまし'		, 'わたまし'				], # 10 甲戌
	  [nil,       nil,    nil,	'出行市買納財安床帳吉', 							 nil			, 'くさかり'				], # 11 乙亥
	  [nil,       nil,    nil,	'斬草吉',									 nil			, nil					], # 12 丙子
	  ['三寶吉',  '神吉', nil,	'療病壊垣破屋吉',								 nil			, nil					], # 13 丁丑
	  ['三寶吉',  nil,    nil,	'裁衣壊垣吉',									 'たねまき'		, 'みそすつくり'			], # 14 戊寅
	  ['三寶吉+', '神吉', nil,	'拜官祠祀結婚納徴剃頭修磑謝土吉',						 nil			, nil					], # 15 己卯
	  ['三寶吉',  nil,    '雑事吉', '祠祀種蒔市買納財吉',								 'けんふく'		, 'けんふく'				], # 16 庚辰
	  ['三寶吉',  nil,    nil,	'拜官冊授療病裁衣市買納財吉',							 'ものたち'		, nil					], # 17 辛巳
	  ['三寶吉',  '神吉', '雑事吉', '剃頭謝土塞穴斬草吉',								 nil			, nil					], # 18 壬午
	  ['三寶吉',  nil,    nil,	'嫁娶出行祀竈吉',								 'たねまき'		, nil					], # 19 癸未
	  ['三寶吉+', '神吉', nil,	'婢娶納婦療病解除除服掃舎葬斬草吉',						 nil			, 'いちたち'				], # 20 甲申
	  ['三寶吉',  '神吉', nil,	'祠祀嫁娶剃頭謝土解除除服経絡安床帳吉', 					 nil			, 'たねまき'				], # 21 乙酉
	  ['三寶吉+', nil,    nil,	'市買納財吉',									 nil			, nil					], # 22 丙戌
	  ['三寶吉+', nil,    nil,	'裁衣市買納財安床帳吉', 							 nil			, nil					], # 23 丁亥
	  [nil,       nil,    nil,	'裁衣吉',									 nil			, nil					], # 24 戊子
	  ['三寶吉+', nil,    nil,	'療病壊垣破屋吉',								 nil			, nil					], # 25 己丑
	  ['三寶吉',  nil,    nil,	'結婚納徴葬斬草吉',								 'やたて'		, 'たねまき'				], # 26 庚寅
	  ['三寶吉+', '神吉', '雑事吉', '拜官納徴納婦出行納頭謝土葬斬草吉',						 nil			, nil					], # 27 辛卯
	  ['三寶吉+', nil,    nil,	'市買納財吉',									 nil			, nil					], # 28 壬辰
	  ['三寶吉',  nil,    nil,	'拜官祠祀冊授療病裁衣祀竈吉',							 'いちたち'		, nil					], # 29 癸巳
	  ['三寶吉',  '神吉', '雑事吉', '結婚納徴剃頭修造吉',								 nil			, nil					], # 30 甲午
	  ['三寶吉+', nil,    nil,	'出行吉',									 nil			, nil					], # 31 乙未
	  ['三寶吉+', '神吉', nil,	nil,										 nil			, nil					], # 32 丙申
	  ['三寶吉',  '神吉', nil,	'除服吉',									 'くさかり'		, nil					], # 33 丁酉
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 34 戊戌
	  ['三寶吉+', nil,    nil,	'裁衣市買吉',									 'ふねのり'		, nil					], # 35 己亥
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 36 庚子
	  ['三寶吉',  '神吉', nil,	nil,										 nil			, nil					], # 37 辛丑
	  ['三寶吉',  nil,    '雑事吉', nil,										 'けんふく'		, nil					], # 38 壬寅
	  ['三寶吉',  '神吉', '雑事吉', '拜官納徴結婚納婦葬斬草吉',							 nil			, nil					], # 39 癸卯
	  ['三寶吉',  nil,    '雑事吉', '種蒔吉',									 'わたまし'		, nil					], # 40 甲辰
	  ['三寶吉+', nil,    nil,	'療病裁衣市買納財吉',								 nil			, nil					], # 41 乙巳
	  [nil,       nil,    nil,	'此日不沐浴',									 nil			, nil					], # 42 丙午
	  [nil,       nil,    nil,	'此日為解除見々乃麻加乃金神',							 nil			, nil					], # 43 丁未
	  ['三寶吉',  '神吉', nil,	'嫁娶納婦療病解除壊垣吉',							 'たねまき'		, nil					], # 44 戊申
	  ['三寶吉',  '神吉', nil,	'祠祀経絡剃頭謝土解除修磑安床帳吉',						 nil			, nil					], # 45 己酉
	  ['三寶吉+', nil,    nil,	'市買納財吉',									 nil			, nil					], # 46 庚戌
	  ['三寶吉',  nil,    nil,	'裁衣市買納財安床帳吉', 							 nil			, nil					], # 47 辛亥
	  [nil,       nil,    nil,	'結婚納徴吉',									 nil			, nil					], # 48 壬子
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 49 癸丑
	  ['三寶吉',  nil,    nil,	'結婚納徴殯埋斬草葬壊垣吉',							 'やたて'		, nil					], # 50 甲寅
	  ['三寶吉',  '神吉', '雑事吉', '結婚納徴剃頭謝土斬草吉',							 nil			, nil					], # 51 乙卯
	  ['三寶吉+', nil,    '雑事吉', '種蒔吉',									 'けんふく'		, nil					], # 52 丙辰
	  [nil,       nil,    nil,	'此日不見死人日乃万加乃神',							 nil			, nil					], # 53 丁巳
	  [nil,       nil,    nil,	'不沐浴三知神', 								 nil			, nil					], # 54 戊午
	  [nil,       nil,    nil,	'不為解除耳又麻加乃金神',							 nil			, nil					], # 55 己未
	  [nil,       '神吉', nil,	'療病掃舎壊垣破屋解除除服葬斬草吉',						 'くさかり'		, 'たねまき くさかり'			], # 56 庚申
	  ['三寶吉',  '神吉', nil,	'祠祀出行経絡解除裁衣市買納財葬斬草安床帳吉',					 'ものたち 井ほり'	, nil					], # 57 辛酉
	  ['三寶吉+', nil,    nil,	'嫁娶市買納財吉',								 nil			, nil					], # 58 壬戌
	  ['三寶吉+', nil,    nil,	'裁衣市買納財奴婢六蓄安床帳吉', 						 nil			, nil					], # 59 癸亥
	],
	[ # 7月
	  ['三寶吉+', nil,    '雑事吉', '加冠拜官冊授謝土起土上梁祠祀修宅門戸碓磑補城郭土坏安床帳吉',			 'ふねのり'		, nil					], #  0 甲子
	  ['三寶吉+', '神吉', nil,	'起土修宅碓磑吉',								 nil			, nil					], #  1 乙丑
	  ['三寶吉',  nil,    nil,	'拜官結婚納徴移徒療病壊垣破屋修宅補城郭壊土起土斬草吉', 			 'けんふく'		, nil					], #  2 丙寅
	  ['三寶吉',  '神吉', '雑事吉', '結婚納徴移徒壊垣破屋修宅宮室碓磑竈殯埋斬草吉', 				 'わたまし'		, nil					], #  3 丁卯
	  ['三寶吉+', nil,    nil,	'入學祀門戸吉', 								 'いちたち'		, nil					], #  4 戊辰
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官祠祀冊授移徒剃頭謝土裁衣市買納財吉',					 nil			, nil					], #  5 己巳
	  ['三寶吉',  '神吉', nil,	'加冠拜官移徒出行療病人學起土上梁修宮室碓磑補城郭塘土吉',			 nil			, nil					], #  6 庚午
	  ['三寶吉',  nil,    nil,	'堤防雍水塞穴吉',								 'ものたち'		, nil					], #  7 辛未
	  ['三寶吉+', '神吉', nil,	'拜官出行修宅宮室坏塘除服解除嫁娶吉',						 nil			, nil					], #  8 壬申
	  [nil,       nil,    nil,	'嫁娶療病解除壊垣破屋葬斬草除服吉',						 nil			, nil					], #  9 癸酉
	  ['三寶吉+', nil,    nil,	'祠祀謝土経絡安床帳吉', 							 'くらたて'		, nil					], # 10 甲戌
	  [nil,       nil,    '雑事吉', '祠祀移徒修磑裁衣市買納財吉',							 nil			, nil					], # 11 乙亥
	  ['三寶吉+', nil,    nil,	'加冠祠祀結婚納徴移徒出行修理補城郭斬草安床帳吉',				 '田かり'		, nil					], # 12 丙子
	  ['三寶吉',  '神吉', nil,	'入學吉',									 nil			, nil					], # 13 丁丑
	  ['三寶吉',  nil,    nil,	'結婚納徴療病壊垣吉',								 'たねまき'		, nil					], # 14 戊寅
	  ['三寶吉+', '神吉', nil,	'加冠結婚納徴移徒壊垣吉',							 'みそすつくり' 	, nil					], # 15 己卯
	  ['三寶吉',  nil,    nil,	'移徒入學起土上梁蓋屋修宅門戸謝土立柱吉',					 'かといて'		, nil					], # 16 庚辰
	  ['三寶吉',  nil,    nil,	'加冠拜官冊授移徒剃頭裁衣市買修宅門戸補城郭殯埋葬斬草吉',			 nil			, nil					], # 17 辛巳
	  ['三寶吉',  '神吉', '雑事吉', '加冠結婚嫁娶移徒出行入學起土上梁修宮室門戸補城坏塘葬斬草吉',			 nil			, nil					], # 18 壬午
	  ['三寶吉',  nil,    nil,	'結婚納徴納婦修宅宮室塞穴補城坏吉',						 nil			, nil					], # 19 癸未
	  ['三寶吉+', '神吉', nil,	'拜官解除出行吉',								 nil			, nil					], # 20 甲申
	  [nil,       nil,    nil,	'此日不裁衣',									 nil			, nil					], # 21 乙酉
	  ['三寶吉+', nil,    nil,	'修宮室宅坏土吉',								 nil			, nil					], # 22 丙戌
	  ['三寶吉+', nil,    nil,	'加冠祠祀裁衣移徒修碓磑市買納財吉',						 nil			, nil					], # 23 丁亥
	  ['三寶吉+', nil,    '雑事吉', '裁衣市買納財安床帳吉', 							 'けんふく くさかり'	, nil					], # 24 戊子
	  ['三寶吉+', nil,    nil,	'壊垣吉',									 nil			, nil					], # 25 己丑
	  ['三寶吉',  nil,    nil,	'結婚納徴療病壊垣破屋吉',							 nil			, nil					], # 26 庚寅
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官結婚納徴破屋斬草壊垣吉',						 nil			, nil					], # 27 辛卯
	  ['三寶吉+', nil,    nil,	'祠祀移徒入學起土蓋屋修宮室門戸補城郭坏壊上梁吉',				 nil			, nil					], # 28 壬辰
	  ['三寶吉',  nil,    '雑事吉', '加冠拜官移徒剃頭裁衣市買納財吉',						 'たねまき'		, 'たねまき'				], # 29 癸巳
	  ['三寶吉',  '神吉', '雑事吉', '加冠拜官納徴嫁娶移徒出行療病入學起土上梁修宅門戸吉',				 'かといて'		, nil					], # 30 甲午
	  ['三寶吉+', nil,    nil,	'起土修宮室宅坏壊雍水塞穴吉',							 nil			, nil					], # 31 乙未
	  ['三寶吉+', '神吉', nil,	'拜官出行解除除服吉',								 '入学'			, '入学'				], # 32 丙申
	  [nil,       nil,    nil,	'療病解除除服入學壊垣破屋吉',							 nil			, nil					], # 33 丁酉
	  [nil,       nil,    nil,	'経絡裁衣市買納財安床帳吉',							 'くらたて かまぬり'	, 'くさかり'				], # 34 戊戌
	  ['三寶吉+', '神吉', '雑事吉', '加冠祠祀裁衣市買納財吉',							 nil			, nil					], # 35 己亥
	  ['三寶吉',  '神吉', '雑事吉', '加冠拜官祠祀修理城郭移徒安床帳吉',						 nil			, nil					], # 36 庚子
	  ['三寶吉',  '神吉', nil,	nil,										 nil			, nil					], # 37 辛丑
	  ['三寶吉',  nil,    '雑事吉', '結婚納徴療病壊垣破屋葬斬草吉', 						 'やたて'		, nil					], # 38 壬寅
	  ['三寶吉',  '神吉', '雑事吉', '結婚納徴壊垣斬草吉',							       'わたまし みそすつくり'	, 'けんふく みそすつくり'		], # 39 癸卯
	  [nil,       nil,    nil,	'此日不見死人', 								 nil			, nil					], # 40 甲辰
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官冊授移徒剃頭市買納財奴婢六蓄吉',					 'けんふく'		, nil					], # 41 乙巳
	  ['三寶吉',  '神吉', '雑事吉', '加冠拜官移徒入學出行療病葬斬草吉',						 'かといて'		, 'かといて'				], # 42 丙午
	  ['三寶吉',  '神吉', nil,	'塞穴吉',									 'いちたち'		, 'いちたち'				], # 43 丁未
	  ['三寶吉',  '神吉', nil,	'加冠拜官祠祀冊授嫁娶出行起土補修城郭坏塘解除除服吉',				 nil			, nil					], # 44 戊申
	  [nil,       nil,    nil,	'祠祀療病解除壊垣破屋葬除服吉', 						 nil			, nil					], # 45 己酉
	  ['三寶吉+', nil,    nil,	'起土修造宮室宅坏土塘安床帳吉', 						 nil			, nil					], # 46 庚戌
	  ['三寶吉',  '神吉', nil,	'加冠拜官冊授祠祀起土竪柱上梁蓋屋修宮室宅門戸碓磑補城坏塘市買納財吉',		 nil			, nil					], # 47 辛亥
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官納婚納徴移徒謝土起土上梁修宅門戸碓磑補坏塘斬草安床帳吉',		 'くらたて'		, 'くさかり'				], # 48 壬子
	  ['三寶吉',  '神吉', nil,	'修宅徒井吉',									 nil			, nil					], # 49 癸丑
	  ['三寶吉',  nil,    nil,	'結婚納徴療病壊垣破屋吉',							 'かといて'		, 'かといて'				], # 50 甲寅
	  ['三寶吉',  '神吉', '雑事吉', '結婚納徴納婦壊垣破屋殯埋斬草吉',						 'くらたて かまぬり'	, 'けんふく'				], # 51 乙卯
	  ['三寶吉+', nil,    nil,	'起土入學吉',									 'くさかり'		, nil					], # 52 丙辰
	  ['三寶吉',  nil,    '雑事吉', '拜官移徒修宮室補城郭坏市買納財吉',						 'ものたち'		, nil					], # 53 丁巳
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官嫁娶移徒出行療病入學補城郭吉', 					 '入学' 		, '入学 いちたち'			], # 54 戊午
	  ['三寶吉',  '神吉', nil,	'塞穴吉',									 'みそすつくり' 	, 'みそすつくり'			], # 55 己未
	  [nil,       nil,    nil,	'此為解除月利神不行病人家',							 nil			, nil					], # 56 庚申
	  [nil,       nil,    nil,	'療病解除除服壊垣破屋葬斬草吉', 						 nil			, nil					], # 57 辛酉
	  ['三寶吉+', nil,    nil,	'拜官上梁修宮室宅屋門戸補城吉', 						 nil			, nil					], # 58 壬戌
	  ['三寶吉+', nil,    nil,	'加冠拜官移徒裁衣市買納財吉',							 nil			, nil					], # 59 癸亥
	],
	[ # 8月
	  ['三寶吉+', nil,    '雑事吉', '加冠拜官冊授祠祀起土上梁蓋屋修宅門戸碓磑井竈築城郭吉', 			 'わたまし'		, 'わたまし'				], #  0 甲子
	  ['三寶吉+', '神吉', nil,	'移徒修宅碓磑井安床帳吉',							 nil			, nil					], #  1 乙丑
	  ['三寶吉',  nil,    nil,	'結婚納徴起土修宅碓磑上梁斬草吉',						 nil			, '田かり'				], #  2 丙寅
	  ['三寶吉',  nil,    nil,	'結婚納徴療病壊破屋斬草吉',							 '田かり'		, '田かり'				], #  3 丁卯
	  ['三寶吉+', nil,    nil,	'壊垣吉',									 'みそすつくり' 	, 'みそすつくり'			], #  4 戊辰
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官冊授移徒祠祀剃頭入學市買納財奴婢六蓄吉',				 nil			, nil					], #  5 己巳
	  [nil,       nil,    nil,	'移徒謝土修宅屋碓磑倉吉',							 '竹木こり'		, '竹木こり'				], #  6 庚午
	  ['三寶吉',  nil,    nil,	'嫁娶移徒入學起土修宅碓磑井倉出行吉',						 nil			, nil					], #  7 辛未
	  ['三寶吉+', '神吉', nil,	'拜官嫁娶剃頭解除塞穴葬斬草除服吉',						 nil			, '田かり'				], #  8 壬申
	  ['三寶吉',  nil,    nil,	'出行解除除服吉',								 nil			, nil					], #  9 癸酉
	  ['三寶吉+', nil,    '雑事吉', '療病解除除服壊垣破屋吉',							 '竹木こり'		, 'かといて'				], # 10 甲戌
	  [nil,       nil,    '雑事吉', '加冠祀門経絡裁衣市買納財奴婢六蓄修宅倉碓磑井安床帳吉', 			 '入学' 		, '入学 いちたち 竹木こり'		], # 11 乙亥
	  ['三寶吉+', nil,    nil,	'加冠拜官祠祀移徒起土竪柱上梁修宅築城郭吉',					 nil			, nil					], # 12 丙子
	  ['三寶吉',  '神吉', nil,	'裁衣市買納財安床帳吉', 							 'ものたち'		, nil					], # 13 丁丑
	  ['三寶吉',  nil,    nil,	'結婚納徴療病壊垣吉',								 '田かり'		, '田かり'				], # 14 戊寅
	  ['三寶吉+', nil,    nil,	'結婚納徴壊垣吉',								 nil			, nil					], # 15 己卯
	  ['三寶吉',  nil,    '雑事吉', '移徒謝土壊垣破屋吉',								 nil			, 'わたまし けんふく'			], # 16 庚辰
	  ['三寶吉',  nil,    nil,	'加冠拜官冊授祠祀嫁娶入學剃頭謝土移徒起土修宅倉築城郭市買吉',			 nil			, nil					], # 17 辛巳
	  [nil,       nil,    nil,	'嫁娶納婦祠祀移徒起土上梁修宅門戸碓磑倉井築城郭吉',				 nil			, nil					], # 18 壬午
	  ['三寶吉',  nil,    nil,	'祠祀嫁娶納徴入學出行起土吉',							 nil			, nil					], # 19 癸未
	  ['三寶吉+', '神吉', nil,	'嫁娶納頭解除塞穴葬斬草除服吉', 						 '竹木こり 田かり'	, '竹木こり 田かり'			], # 20 甲申
	  ['三寶吉',  nil,    nil,	'出行解除吉',									 nil			, nil					], # 21 乙酉
	  ['三寶吉+', nil,    nil,	'療病解除壊垣破屋吉',								 nil			, nil					], # 22 丙戌
	  ['三寶吉+', nil,    nil,	'加冠祀門移徒経絡市買納財安床帳吉',						 '入学' 		, '入学'				], # 23 丁亥
	  ['三寶吉+', nil,    '雑事吉', '加冠謝土祠祀吉',								 nil			, 'けんふく'				], # 24 戊子
	  ['三寶吉+', nil,    nil,	'裁衣市買納財安床帳吉', 							 'かまぬり'		, nil					], # 25 己丑
	  ['三寶吉',  nil,    nil,	'結婚納徴葬斬草吉',								 nil			, nil					], # 26 庚寅
	  ['三寶吉+', nil,    nil,	'結婚納徴療病壊垣破屋吉',							 '田かり'		, nil					], # 27 辛卯
	  ['三寶吉+', nil,    nil,	'壊垣破屋吉',									 nil			, nil					], # 28 壬辰
	  ['三寶吉',  nil,    '雑事吉', '加冠拜官嫁娶移徒剃頭入學謝土修倉吉',						 nil			, nil					], # 29 癸巳
	  [nil,       nil,    nil,	'加冠拜官祠祀嫁娶納徴納婦修理宅屋上梁吉',					 nil			, '田かり'				], # 30 甲午
	  ['三寶吉+', nil,    nil,	'療病移徒出行入學吉',								 nil			, nil					], # 31 乙未
	  ['三寶吉+', '神吉', nil,	'剃頭解除除服塞穴斬草吉',							 nil			, '竹木こり'				], # 32 丙申
	  ['三寶吉',  nil,    nil,	'出行解除除服吉',								 nil			, nil					], # 33 丁酉
	  [nil,       nil,    nil,	'移徒療病解除修碓磑井壊垣吉',							 'ふねのり 田かり'	, nil					], # 34 戊戌
	  ['三寶吉+', '神吉', '雑事吉', '加冠祠祀経絡市買納財安床帳吉', 						 'いちたち'		, nil					], # 35 己亥
	  ['三寶吉',  '神吉', '雑事吉', '加冠拜官冊授移徒謝土上梁蓋屋修宅碓磑倉築城郭吉',				 nil			, nil					], # 36 庚子
	  ['三寶吉',  '神吉', nil,	'裁衣市買納財安床帳吉', 							 'ものたち'		, nil					], # 37 辛丑
	  ['三寶吉',  nil,    nil,	'結婚納徴葬斬草吉',								 'やたて'		, nil					], # 38 壬寅
	  ['三寶吉',  nil,    nil,	'結婚納徴壊垣破屋吉',								 'くらたて 田かり'	, nil					], # 39 癸卯
	  ['三寶吉',  nil,    '雑事吉', '壊垣吉',									 'けんふく わたまし'	, nil					], # 40 甲辰
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官冊授移徒剃頭入學裁衣市買納財奴婢六蓄吉',				 nil			, nil					], # 41 乙巳
	  [nil,       nil,    nil,	'祀門移徒修移倉築城郭吉',							 nil			, nil					], # 42 丙午
	  ['三寶吉',  '神吉', nil,	'出行入學起土吉',								 nil			, nil					], # 43 丁未
	  ['三寶吉',  '神吉', nil,	'祠祀嫁娶納頭解除竪柱上梁修宅倉井築城郭塞穴吉', 				 nil			, nil					], # 44 戊申
	  [nil,       nil,    nil,	'此日不拂倉内阿知神不嫁娶不家内動之衝死人神',					 nil			, nil					], # 45 己酉
	  ['三寶吉+', nil,    nil,	'療病解除壊垣破屋起土吉',							 nil			, nil					], # 46 庚戌
	  ['三寶吉',  '神吉', nil,	'加冠祠祀冊授結婚経絡謝土移徒起土修宅碓磑築城郭納財六蓄安床帳吉',		 nil			, nil					], # 47 辛亥
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官祠祀結婚納徴移徒起土上梁修宅門戸碓磑倉城郭吉', 			 nil			, nil					], # 48 壬子
	  ['三寶吉',  '神吉', nil,	'移徒起土市買納財安床帳吉',							 'たねまき 竹木こり'	, nil					], # 49 癸丑
	  ['三寶吉',  nil,    nil,	'結婚納徴殯埋斬草吉',								 'かまぬり 田かり'	, nil					], # 50 甲寅
	  [nil,       nil,    nil,	'此日不見死人焼雷神不同産家不殺木',						 nil			, nil					], # 51 乙卯
	  ['三寶吉+', nil,    '雑事吉', '壊垣破屋吉',									 nil			, nil					], # 52 丙辰
	  ['三寶吉',  nil,    '雑事吉', '加冠拜官移徒入學市買納財吉',							 nil			, nil					], # 53 丁巳
	  [nil,       nil,    nil,	'祠祀結婚納徴嫁娶納婦移徒謝土吉',						 nil			, nil					], # 54 戊午
	  ['三寶吉',  '神吉', nil,	'出行入學移徒吉',								 nil			, nil					], # 55 己未
	  [nil,       '神吉', nil,	'拜官剃頭解除除服塞穴葬斬草吉', 						 '入学' 		, nil					], # 56 庚申
	  [nil,       nil,    nil,	'此日為解除耳乃買神不行病人家', 						 nil			, nil					], # 57 辛酉
	  ['三寶吉+', nil,    nil,	'移徒療病解除壊垣屋吉', 							 'たねまき'		, 'たねまき'				], # 58 壬戌
	  ['三寶吉+', nil,    nil,	'加冠拜官祠祀経絡裁衣修宅門戸倉築城郭市買納財安床帳吉', 			 'いちたち'		, 'いちたち'				], # 59 癸亥
	],
	[ # 9月
	  ['三寶吉+', nil,    nil,	'拜官祠祀経絡竪柱上梁蓋屋修宅門戸碓磑井安床帳起土吉',				 nil			, nil					], #  0 甲子
	  ['三寶吉+', '神吉', nil,	'裁衣市買納財吉',								 nil			, nil					], #  1 乙丑
	  [nil,       nil,    nil,	'此日不為淨事天一神不話談',							 nil			, nil					], #  2 丙寅
	  ['三寶吉',  '神吉', '雑事吉', '結婚納徴起土修宅門戸碓磑祀門侯猟漁殯埋斬草吉', 				 '田かり'		, 'かといて 田かり'			], #  3 丁卯
	  ['三寶吉+', nil,    nil,	'療病壊垣吉',									 nil			, nil					], #  4 戊辰
	  ['三寶吉+', '神吉', '雑事吉', '拜官裁衣市買納財壊垣吉',							 nil			, nil					], #  5 己巳
	  ['三寶吉',  '神吉', nil,	'拜官祠祀結婚納婦嫁娶剃頭謝土上梁修宅門戸碓磑葬斬草吉', 			 'みそすつくり' 	, nil					], #  6 庚午
	  ['三寶吉',  nil,    nil,	'嫁娶磑去吉',									 'たねまき 田かり'	, nil					], #  7 辛未
	  ['三寶吉+', nil,    nil,	'祠祀出行療病解除除服吉',							 'むきまき'		, nil					], #  8 壬申
	  ['三寶吉',  '神吉', nil,	'塞穴解除除服葬斬草吉', 							 nil			, nil					], #  9 癸酉
	  [nil,       nil,    nil,	'此日不可病起納吉神不種云起',							 nil			, nil					], # 10 甲戌
	  [nil,       nil,    '雑事吉', '療病解除除服掃舎壊垣破屋吉',							 '田かり'		, nil					], # 11 乙亥
	  ['三寶吉+', nil,    nil,	'拜官経絡起土上梁修宅門戸碓磑井斬草安床帳吉',					 nil			, nil					], # 12 丙子
	  ['三寶吉',  '神吉', nil,	'裁衣市買納財吉',								 nil			, nil					], # 13 丁丑
	  [nil,       nil,    nil,	'此日不為降事', 								 nil			, nil					], # 14 戊寅
	  ['三寶吉+', '神吉', nil,	'結婚納徴剃頭謝土起土修宅門戸磑吉',						 '竹木こり'		, nil					], # 15 己卯
	  ['三寶吉',  nil,    '雑事吉', '嫁娶療病壊垣破屋吉',								 nil			, nil					], # 16 庚辰
	  ['三寶吉',  nil,    nil,	'拜官嫁娶裁衣市買納財壊垣破屋吉',						 nil			, nil					], # 17 辛巳
	  ['三寶吉',  '神吉', '雑事吉', '拜官冊授祠祀嫁娶納婦剃頭謝土修宅門戸碓磑井葬斬草吉',				 'けんふく'		, nil					], # 18 壬午
	  ['三寶吉',  nil,    nil,	'納徴嫁娶磑吉', 								 'むきまき 田かり'	, nil					], # 19 癸未
	  ['三寶吉+', nil,    nil,	'出行療病解除葬斬移除服吉',							 nil			, nil					], # 20 甲申
	  ['三寶吉',  '神吉', nil,	'解除除服祀門塞穴吉',								 '田かり'		, nil					], # 21 乙酉
	  ['三寶吉+', nil,    nil,	'出行吉',									 '竹木こり'		, nil					], # 22 丙戌
	  ['三寶吉+', nil,    nil,	'療病解除掃舎市買納財吉',							 nil			, nil					], # 23 丁亥
	  ['三寶吉+', nil,    nil,	'祠祀謝土経絡安床帳吉', 							 nil			, nil					], # 24 戊子
	  ['三寶吉+', nil,    nil,	'裁衣市買納財吉',								 nil			, nil					], # 25 己丑
	  [nil,       nil,    nil,	'諸種破厭夜乃日神不夜廻',							 nil			, nil					], # 26 庚寅
	  [nil,       nil,    nil,	'不呼婦霹靂神不逍水不置牛馬植木',						 nil			, nil					], # 27 辛卯
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 28 壬辰
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 29 癸巳
	  [nil,       nil,    nil,	nil,										 nil			, '田かり'				], # 30 甲午
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 31 乙未
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 32 丙申
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 33 丁酉
	  [nil,       nil,    nil,	nil,										 '田かり'		, '田かり'				], # 34 戊戌
	  ['三寶吉+', '神吉', '雑事吉', '療病解除壊垣市買納財吉',							 'むきまき'		, 'たねまき'				], # 35 己亥
	  ['三寶吉',  '神吉', nil,	'拜官祠祀修理上梁謝土斬草安床帳吉',						 nil			, nil					], # 36 庚子
	  ['三寶吉',  '神吉', nil,	nil,										 nil			, nil					], # 37 辛丑
	  [nil,       nil,    nil,	'万目防乃神不祠祀解除', 							 nil			, nil					], # 38 壬寅
	  ['三寶吉',  '神吉', '雑事吉', '拜官結婚納徴剃頭猟殯埋斬草吉', 						 'けんふく かといて'	, '田かり'				], # 39 癸卯
	  ['三寶吉',  nil,    '雑事吉', '療病壊垣吉',									 nil			, nil					], # 40 甲辰
	  ['三寶吉+', '神吉', '雑事吉', '拜官裁衣市買納財壊垣吉',							 nil			, nil					], # 41 乙巳
	  ['三寶吉',  '神吉', '雑事吉', '祠祀剃頭起土上梁修宅門戸碓磑井葬斬草吉',					 'いちたち'		, 'わたまし'				], # 42 丙午
	  ['三寶吉',  '神吉', nil,	'猟吉', 									 '田かり'		, 'かといて 田かり'			], # 43 丁未
	  ['三寶吉',  nil,    nil,	'出行療病解除起土上梁修宅門戸碓磑井葬斬草吉',					 nil			, nil					], # 44 戊申
	  ['三寶吉',  '神吉', nil,	'起土解除塞穴吉',								 'たねまき'		, 'むきまき'				], # 45 己酉
	  [nil,       nil,    nil,	'為解除耳万買不行病人家',							 nil			, nil					], # 46 庚戌
	  ['三寶吉',  '神吉', nil,	'祠祀療病解除謝土掃舎修宅碓磑井市買納財吉',					 '田かり'		, 'いちたち 田かり'			], # 47 辛亥
	  ['三寶吉+', '神吉', nil,	'拜官冊授上梁修宅門戸安床帳吉', 						 nil			, nil					], # 48 壬子
	  ['三寶吉',  '神吉', nil,	'裁衣市買納財吉',								 nil			, nil					], # 49 癸丑
	  [nil,       nil,    nil,	'不為淨事不見死人',								 nil			, nil					], # 50 甲寅
	  ['三寶吉',  '神吉', '雑事吉', '結婚納婦剃頭猟殯埋斬草吉',							 'かまぬり ものたち'	, '田かり'				], # 51 乙卯
	  ['三寶吉+', nil,    '雑事吉', '療病壊垣破屋吉',								 'かとたて'		, 'けんふく'				], # 52 丙辰
	  ['三寶吉',  nil,    '雑事吉', '裁衣市買納財壊垣破屋吉',							 nil			, nil					], # 53 丁巳
	  ['三寶吉+', '神吉', '雑事吉', '拜官冊授嫁娶剃頭起土吉',							 nil			, nil					], # 54 戊午
	  ['三寶吉',  '神吉', nil,	'裁衣磑吉',									 'かといて'		, 'かといて'				], # 55 己未
	  [nil,       nil,    nil,	'拜官祠祀出行療病解除謝土吉',							 nil			, '竹木こり'				], # 56 庚申
	  ['三寶吉',  '神吉', nil,	'塞穴解除除服葬斬草吉', 							 'いちたち'		, 'みそすつくり'			], # 57 辛酉
	  ['三寶吉+', nil,    nil,	'出行吉',									 '田かり'		, '田かり'				], # 58 壬戌
	  ['三寶吉+', nil,    nil,	'療病解除掃舎裁衣市買納財吉',							 'むきまき'		, 'むきまき'				], # 59 癸亥
	],
	[ # 10月
	  ['三寶吉+', nil,    '雑事吉', '加冠拜官冊授療病解除起土謝土修宅門戸碓磑井廁塘城郭吉', 			 nil			, nil					], #  0 甲子
	  [nil,       nil,    nil,	nil,										 nil			, nil					], #  1 乙丑
	  ['三寶吉',  nil,    nil,	'加冠拜官冊授結婚納婦移徒起土上梁修宅塘城郭吉', 				 nil			, nil					], #  2 丙寅
	  ['三寶吉',  '神吉', '雑事吉', '加冠拜官結婚納徴起土上梁修宅廁塘城郭斬草安床帳移徒吉', 			 nil			, nil					], #  3 丁卯
	  ['三寶吉+', nil,    nil,	'伐樹猟吉',									 'ふねのり'		, nil					], #  4 戊辰
	  [nil,       nil,    nil,	'不殺木内奴婢', 								 nil			, nil					], #  5 己巳
	  ['三寶吉',  '神吉', nil,	'祠祀嫁娶壊垣破屋伐樹斬草吉',							 'みそさけつくり'	, 'みそさけつくり'			], #  6 庚午
	  ['三寶吉',  nil,    nil,	'入學吉',									 'やたて ものたち'	, nil					], #  7 辛未
	  ['三寶吉+', '神吉', nil,	'拜官移徒剃頭解除起土修宅井廁塘城郭伐樹磑種蒔吉',				 nil			, nil					], #  8 壬申
	  ['三寶吉',  '神吉', nil,	'拜官出行療病入學謝土解除除服起土上梁吉',					 nil			, nil					], #  9 癸酉
	  ['三寶吉+', nil,    '雑事吉', '塞穴吉',									 'わたまし いちたち'	, 'わたまし みそさけつくり'		], # 10 甲戌
	  [nil,       nil,    nil,	'市買納財吉',									 '入学' 		, '入学 木こり'				], # 11 乙亥
	  ['三寶吉+', nil,    nil,	'拜官結婚納徴療病解除壊垣破屋吉',						 nil			, 'たねまき むきまき'			], # 12 丙子
	  [nil,       nil,    nil,	'不為淨事',									 nil			, nil					], # 13 丁丑
	  ['三寶吉',  nil,    nil,	'加冠拜官結婚納徴納婦移徒吉',							 nil			, nil					], # 14 戊寅
	  ['三寶吉+', '神吉', nil,	'加冠拜官祠祀結婚納徴嫁娶納婦移徒出行起土修宅門戸廁吉', 			 nil			, nil					], # 15 己卯
	  ['三寶吉',  nil,    '雑事吉', '嫁娶起土伐樹猟吉',								 'けんふく よめとり'	, 'かといて'				], # 16 庚辰
	  ['三寶吉',  nil,    nil,	'拜官冊授療病市買納財壊垣破屋吉',						 nil			, nil					], # 17 辛巳
	  ['三寶吉',  '神吉', '雑事吉', '嫁娶納婦移徒壊垣破屋伐樹吉',							 'かとたて'		, 'むきまき'				], # 18 壬午
	  ['三寶吉',  nil,    nil,	'入學吉',									 nil			, 'みそさけつくり'			], # 19 癸未
	  ['三寶吉+', '神吉', nil,	'祠祀移徒剃頭解除修宅起土上梁壊城郭伐樹除服吉', 				 nil			, nil					], # 20 甲申
	  ['三寶吉',  '神吉', nil,	'祠祀移徒出行療病人學起土修宅門戸碓磑井廁壊城郭解除吉', 			 nil			, nil					], # 21 乙酉
	  ['三寶吉+', nil,    nil,	'塞穴吉',									 nil			, nil					], # 22 丙戌
	  [nil,       nil,    nil,	'裁衣市買納財吉',								 nil			, nil					], # 23 丁亥
	  [nil,       nil,    nil,	'不祈願',									 'たねまき'		, 'たねまき'				], # 24 戊子
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 25 己丑
	  ['三寶吉',  nil,    nil,	'加冠拜官移徒吉',								 'かといて'		, 'かといて'				], # 26 庚寅
	  ['三寶吉+', '神吉', '雑事吉', '加冠拜官結婚納徴嫁娶納婦移徒剃頭修理門戸斬草安床帳吉', 			 'ものたち'		, 'よめとり'				], # 27 辛卯
	  ['三寶吉+', nil,    nil,	'嫁娶伐樹猟吉', 								 nil			, nil					], # 28 壬辰
	  ['三寶吉',  nil,    '雑事吉', '拜官療病裁衣市買納財壊垣吉',							 nil			, 'けんふく'				], # 29 癸巳
	  ['三寶吉',  '神吉', '雑事吉', '移徒壊垣伐樹斬草吉',								 'かまぬり むきまき'	, nil					], # 30 甲午
	  ['三寶吉+', nil,    nil,	'入學吉',									 'みそさけつくり 木こり', 'みそさけつくり 木こり'		], # 31 乙未
	  ['三寶吉+', '神吉', nil,	'祠祀移徒剃頭解除起土修宅門戸碓磑井廁坏城伐樹吉',				 nil			, nil					], # 32 丙申
	  ['三寶吉',  '神吉', nil,	'祠祀移徒出行療病人學解除除服起土上梁修宅門戸碓磑吉',				 nil			, nil					], # 33 丁酉
	  [nil,       nil,    nil,	'不呼婦不殖木不買牛馬', 							 nil			, nil					], # 34 戊戌
	  [nil,       nil,    nil,	'不呼婦',									 nil			, nil					], # 35 己亥
	  ['三寶吉',  '神吉', '雑事吉', '拜官療病解除除服壊垣斬草吉',							 'くらたて 井ほり'	, nil					], # 36 庚子
	  [nil,       nil,    nil,	'諸種破厭',									 nil			, nil					], # 37 辛丑
	  ['三寶吉',  nil,    '雑事吉', '拜官納徴結婚移徒上梁修碓磑井坏塘城郭吉',					 nil			, nil					], # 38 壬寅
	  ['三寶吉',  '神吉', '雑事吉', '加冠結婚納徴嫁娶納婦剃頭謝土斬草安床帳吉',					 nil			, nil					], # 39 癸卯
	  ['三寶吉',  nil,    '雑事吉', '伐樹猟吉',									 'やたて かとたて'	, nil					], # 40 甲辰
	  ['三寶吉+', '神吉', '雑事吉', '療病壊垣破屋裁衣市買納財吉',							 nil			, nil					], # 41 乙巳
	  ['三寶吉',  '神吉', '雑事吉', '壊垣伐樹吉',									 nil			, nil					], # 42 丙午
	  ['三寶吉',  nil,    nil,	'入學吉',									 '木こり'		, nil					], # 43 丁未
	  ['三寶吉',  '神吉', nil,	'移徒起土修宅門戸碓磑井廁坏城剃頭伐樹解除除服吉',				 nil			, nil					], # 44 戊申
	  ['三寶吉',  '神吉', nil,	'加冠祠祀出行療病解除除服上梁修宅門戸葬斬草吉', 				 nil			, nil					], # 45 己酉
	  ['三寶吉+', nil,    nil,	'塞穴吉',									 'たねまき'		, nil					], # 46 庚戌
	  [nil,       nil,    nil,	'加冠起土修宅坏城郭吉', 							 nil			, nil					], # 47 辛亥
	  [nil,       nil,    nil,	'不誓願',									 nil			, nil					], # 48 壬子
	  [nil,       nil,    nil,	'不殺木不見死人',								 nil			, nil					], # 49 癸丑
	  ['三寶吉',  nil,    nil,	'拜官冊授結婚納徴出行起土上梁修宅門戸碓磑井廁坏城郭吉', 			 'ふねのり'		, nil					], # 50 甲寅
	  ['三寶吉',  '神吉', '雑事吉', '結婚納徴移徒剃頭殯埋斬草安床帳吉',						 'けんふく ものたち'	, nil					], # 51 乙卯
	  ['三寶吉+', nil,    '雑事吉', '伐樹猟吉',									 'かといて'		, nil					], # 52 丙辰
	  [nil,       nil,    nil,	'不為嫁娶不徴財',								 nil			, nil					], # 53 丁巳
	  ['三寶吉+', '神吉', '雑事吉', '壊垣伐樹吉',									 'むきまき'		, nil					], # 54 戊午
	  ['三寶吉',  nil,    nil,	'入學吉',									 'かまぬり いちたち'	, nil					], # 55 己未
	  [nil,       '神吉', nil,	'祠祀移徒剃頭解除起土修宅門戸碓磑井廁坏城伐樹猟種蒔吉', 			 nil			, nil					], # 56 庚申
	  ['三寶吉',  '神吉', nil,	'祠祀出行移徒療病人學起土上梁修宅門戸廁坏城離土葬斬草解除除服吉',		 nil			, nil					], # 57 辛酉
	  ['三寶吉+', nil,    nil,	'塞穴吉',									 'むきまき'		, 'むきまき'				], # 58 壬戌
	  [nil,       nil,    nil,	'不為解除',									 nil			, nil					], # 59 癸亥
	],
	[ # 11月
	  ['三寶吉+', nil,    nil,	'加冠拜官冊授移徒謝土修宅門戸碓磑廁蓋吉',					 nil			, nil					], #  0 甲子
	  ['三寶吉+', '神吉', nil,	'療病解除壊垣破屋吉',								 'やたて'		, nil					], #  1 乙丑
	  ['三寶吉',  nil,    nil,	'加冠拜官冊授納徴経絡修宅門戸廁葬斬草安床帳吉', 				 nil			, 'みそさけつくり'			], #  2 丙寅
	  ['三寶吉',  '神吉', '雑事吉', '加冠拜官冊授結婚納徴嫁娶納婦出行移徒上梁蓋屋門戸吉',				 nil			, nil					], #  3 丁卯
	  ['三寶吉+', nil,    nil,	'裁衣市買納財婦婢六蓄安床帳吉', 						 nil			, nil					], #  4 戊辰
	  ['三寶吉+', '神吉', '雑事吉', '拜官祠祀嫁娶裁衣市買納財吉',							 nil			, nil					], #  5 己巳
	  ['三寶吉',  nil,    nil,	'療病壊垣破屋伐樹殯埋斬草吉',							 nil			, nil					], #  6 庚午
	  ['三寶吉',  nil,    nil,	'壊垣破屋吉',									 'かとたて'		, nil					], #  7 辛未
	  [nil,       nil,    nil,	'拜官冊授剃頭入學解除除服起土修門戸碓磑廁伐樹葬斬草吉', 			 'いちたち'		, nil					], #  8 壬申
	  ['三寶吉',  '神吉', nil,	'拜官移徒修宅門戸廁猟解除除服吉',						 nil			, nil					], #  9 癸酉
	  ['三寶吉+', nil,    '雑事吉', '出行入學療病吉',								 nil			, nil					], # 10 甲戌
	  [nil,       nil,    '雑事吉', nil,										 nil			, nil					], # 11 乙亥
	  ['三寶吉+', nil,    nil,	'加冠拜官祠祀結婚納徴吉',							 '木こり'		, nil					], # 12 丙子
	  ['三寶吉',  '神吉', nil,	'療病解除壊垣破屋吉',								 'ものたち'		, nil					], # 13 丁丑
	  ['三寶吉',  nil,    nil,	'拜官結婚納徴経絡安床帳吉',							 'みそさけつくり'	, nil					], # 14 戊寅
	  ['三寶吉+', '神吉', nil,	'加冠拜官冊授結婚納徴嫁娶納婦移徒剃頭謝土修宅門戸碓磑廁吉',			 nil			, nil					], # 15 己卯
	  ['三寶吉',  nil,    '雑事吉', '嫁娶納婦安床帳吉',								 'たねまき'		, nil					], # 16 庚辰
	  ['三寶吉',  nil,    nil,	'加冠拜官冊授嫁娶納婦移徒上梁修宅門戸碓磑伐樹吉',				 nil			, nil					], # 17 辛巳
	  ['三寶吉',  nil,    nil,	'祠祀療病壊垣破屋伐樹葬斬草吉', 						 nil			, nil					], # 18 壬午
	  ['三寶吉',  nil,    nil,	'壊垣破屋吉',									 nil			, nil					], # 19 癸未
	  [nil,       nil,    nil,	'祠祀入學剃頭解除伐樹謝土修宅屋門戸碓磑廁斬草除服葬吉', 			 nil			, nil					], # 20 甲申
	  ['三寶吉',  '神吉', nil,	'移徒謝土上梁修宅門戸碓磑廁猟解除吉',						 'ものたち'		, nil					], # 21 乙酉
	  ['三寶吉+', nil,    nil,	'出行入學療病種蒔吉',								 nil			, nil					], # 22 丙戌
	  ['三寶吉+', nil,    nil,	'裁衣市買納財塞穴吉',								 nil			, nil					], # 23 丁亥
	  [nil,       nil,    nil,	'不為誓願',									 nil			, nil					], # 24 戊子
	  ['三寶吉+', nil,    nil,	'療病解除壊垣吉',								 nil			, nil					], # 25 己丑
	  ['三寶吉',  nil,    nil,	'加冠拜官結婚納徴葬斬草安床帳吉',						 'みそさけつくり'	, nil					], # 26 庚寅
	  ['三寶吉+', '神吉', '雑事吉', '拜官結婚納婦嫁娶移徒納頭修理舎屋謝土斬移吉',					 nil			, nil					], # 27 辛卯
	  ['三寶吉+', nil,    nil,	'嫁娶納婦安床帳吉',								 '木こり'		, '木こり'				], # 28 壬辰
	  ['三寶吉',  nil,    '雑事吉', '拜官裁衣市買納財伐樹猟吉',							 nil			, nil					], # 29 癸巳
	  ['三寶吉',  nil,    nil,	'療病壊垣破服伐樹斬草吉',							 nil			, nil					], # 30 甲午
	  ['三寶吉+', nil,    nil,	'壊垣破屋吉',									 'ものたち'		, nil					], # 31 乙未
	  [nil,       nil,    nil,	'祠祀入學移徒納頭解除伐樹殯埋除服斬草上梁修宅門戸碓磑吉',			 nil			, '入学'				], # 32 丙申
	  ['三寶吉',  '神吉', nil,	'移徒裁衣解除修宅門戸猟吉',							 nil			, nil					], # 33 丁酉
	  [nil,       nil,    nil,	'出行入學療病種蒔吉',								 'くらたて かまぬり'	, nil					], # 34 戊戌
	  ['三寶吉+', '神吉', '雑事吉', '裁衣市買納財塞穴吉',								 'ふねのり'		, nil					], # 35 己亥
	  ['三寶吉',  nil,    nil,	'加冠拜官出行入學吉',								 '井ほり'		, nil					], # 36 庚子
	  ['三寶吉',  '神吉', nil,	'療病解除壊垣吉',								 nil			, 'たねまき'				], # 37 辛丑
	  ['三寶吉',  nil,    nil,	'加冠拜官結婚納徴上梁修宅門戸碓磑服葬斬草安床帳吉',				 nil			, nil					], # 38 壬寅
	  ['三寶吉',  '神吉', '雑事吉', '加冠拜官結婚納徴納頭謝土吉',							 nil			, nil					], # 39 癸卯
	  ['三寶吉',  nil,    '雑事吉', '裁衣市買納財安床帳吉', 							 nil			, nil					], # 40 甲辰
	  ['三寶吉+', '神吉', '雑事吉', '拜官裁衣市買納財伐樹吉',							 nil			, nil					], # 41 乙巳
	  [nil,       nil,    nil,	'不打人不動財', 								 nil			, nil					], # 42 丙午
	  ['三寶吉',  '神吉', nil,	'壊垣吉',									 'いちたち'		, 'いちたち'				], # 43 丁未
	  [nil,       nil,    nil,	'祠祀入學納頭移徒伐樹解除除服吉',						 '入学' 		, '入学 みそさけつくり'			], # 44 戊申
	  ['三寶吉',  '神吉', nil,	'拜官移徒上梁修宅門戸碓磑廁解除除服吉', 					 'やたて'		, nil					], # 45 己酉
	  ['三寶吉+', nil,    nil,	'出行入學療病種蒔吉',								 'たねまき'		, 'いちたち'				], # 46 庚戌
	  ['三寶吉',  '神吉', nil,	'裁衣市買納財塞穴吉',								 nil			, nil					], # 47 辛亥
	  [nil,       nil,    nil,	'不為誓願不動財',								 nil			, nil					], # 48 壬子
	  ['三寶吉',  '神吉', nil,	'療病解除壊垣破屋吉',								 '木こり'		, 'たねまき'				], # 49 癸丑
	  ['三寶吉',  nil,    nil,	'加冠拜官結婚納財経絡斬草安床帳吉',						 nil			, '木こり'				], # 50 甲寅
	  ['三寶吉',  '神吉', '雑事吉', '加冠拜官祠祀結婚納財移徒納頭謝土修理宅門戸殯埋斬草吉', 			 nil			, nil					], # 51 乙卯
	  ['三寶吉+', nil,    '雑事吉', '裁衣市買納財安床帳吉', 							 'けんふく かといて'	, 'けんふく かといて'			], # 52 丙辰
	  ['三寶吉',  nil,    '雑事吉', '加冠拜官冊授娶裁衣市買納徴伐樹猟解除吉',					 nil			, nil					], # 53 丁巳
	  ['三寶吉+', nil,    nil,	'療病壊垣伐樹吉',								 nil			, nil					], # 54 戊午
	  ['三寶吉',  '神吉', nil,	'壊垣吉',									 nil			, nil					], # 55 己未
	  [nil,       nil,    nil,	'祠祀移徒入學納頭解除除服伐樹祀井葬斬草吉',					 '入学 みそさけつくり'	, '入学 みそさけつくり'			], # 56 庚申
	  ['三寶吉',  '神吉', nil,	'移徒謝土竪柱上梁修宅門戸碓磑廁猟解除除服吉',					 '井ほり'		, nil					], # 57 辛酉
	  ['三寶吉+', nil,    nil,	'出行入學療病種蒔吉',								 nil			, nil					], # 58 壬戌
	  ['三寶吉+', nil,    nil,	'裁衣市買納財塞穴吉',								 nil			, nil					], # 59 癸亥
	],
	[ # 12月
	  ['三寶吉+', nil,    nil,	'祠祀竪柱上梁修宅門戸碓磑服塞穴吉',						 nil			, nil					], #  0 甲子
	  ['三寶吉+', '神吉', nil,	'出行吉',									 nil			, nil					], #  1 乙丑
	  ['三寶吉',  nil,    nil,	'拜官嫁娶納婦謝土解除掃舎殯埋斬草除服吉',					 'けんふく'		, nil					], #  2 丙寅
	  ['三寶吉',  '神吉', '雑事吉', '拜官冊授祠祀結婚嫁娶修宅門戸碓磑殯埋斬草吉',					 nil			, 'けんふく よめとり'			], #  3 丁卯
	  ['三寶吉+', nil,    nil,	'裁衣市買納財吉',								 nil			, nil					], #  4 戊辰
	  [nil,       nil,    nil,	'裁衣市買納財吉',								 nil			, nil					], #  5 己巳
	  ['三寶吉',  '神吉', nil,	'納頭修碓磑廁伐樹猟殯埋斬草吉', 						 'すゝはらひ'		, 'たねまき'				], #  6 庚午
	  ['三寶吉',  nil,    nil,	'壊垣破屋吉',									 'かといて'		, 'かといて'				], #  7 辛未
	  ['三寶吉+', '神吉', nil,	'祠祀壊垣屋伐樹解除葬斬草除服吉',						 nil			, nil					], #  8 壬申
	  ['三寶吉',  '神吉', nil,	'出行謝土解除除服祀井葬斬草吉', 						 nil			, nil					], #  9 癸酉
	  ['三寶吉+', nil,    '雑事吉', '漁猟種蒔吉',									 'わたまし'		, 'わたまし たねまき'			], # 10 甲戌
	  [nil,       nil,    nil,	'療病修宅門戸碓磑廁裁衣市買納財吉',						 'いちたち'		, 'いちたち'				], # 11 乙亥
	  ['三寶吉+', nil,    nil,	'結婚塞穴斬草納財吉',								 nil			, nil					], # 12 丙子
	  ['三寶吉',  '神吉', nil,	'嫁娶納婦出行吉',								 nil			, nil					], # 13 丁丑
	  ['三寶吉',  nil,    nil,	'拜官結婚納徴納婦療病壊垣吉',							 '木こり すゝはらひ'	, '木こり'				], # 14 戊寅
	  ['三寶吉+', '神吉', nil,	'拜官嫁娶斬頭納婦経絡謝土祀井上梁修宅門戸碓磑廁安床帳吉',			 '正月ことはしめ'	, nil					], # 15 己卯
	  ['三寶吉',  nil,    '雑事吉', '嫁娶納婦吉',									 'よめとり'		, 'けんふく'				], # 16 庚辰
	  [nil,       nil,    nil,	'拜官冊授上梁修宅門戸裁衣市買納財安床帳吉',					 'すゝはらひ'		, nil					], # 17 辛巳
	  ['三寶吉',  '神吉', '雑事吉', '拜官納頭謝土伐樹漁猟上梁修宅門戸碓磑納葬斬草吉',				 '正月ことはしめ'	, nil					], # 18 壬午
	  ['三寶吉',  nil,    nil,	'壊垣破屋吉',									 'たねまき'		, 'たねまき'				], # 19 癸未
	  ['三寶吉+', '神吉', nil,	'祠祀壊垣破屋解除除服伐樹葬斬草吉',						 nil			, nil					], # 20 甲申
	  ['三寶吉',  '神吉', nil,	'修宅門戸碓磑納解除吉', 							 nil			, nil					], # 21 乙酉
	  ['三寶吉+', nil,    nil,	'漁猟種蒔吉',									 nil			, nil					], # 22 丙戌
	  ['三寶吉+', nil,    nil,	'療病市買納財吉',								 nil			, nil					], # 23 丁亥
	  [nil,       nil,    nil,	nil,										 nil			, nil					], # 24 戊子
	  ['三寶吉+', nil,    nil,	'出行嫁娶裁衣吉',								 nil			, nil					], # 25 己丑
	  ['三寶吉',  nil,    nil,	'拜官結婚納徴嫁娶壊垣破除除服掃舎療病斬草吉',					 '井ほり'		, nil					], # 26 庚寅
	  ['三寶吉+', '神吉', '雑事吉', '拜官祠祀結婚納婦嫁娶斬草安床帳吉',						 'よめとり'		, nil					], # 27 辛卯
	  ['三寶吉+', nil,    nil,	'裁衣市買納財吉',								 nil			, nil					], # 28 壬辰
	  [nil,       nil,    nil,	'拜官市買納財裁衣安床帳吉',							 nil			, nil					], # 29 癸巳
	  ['三寶吉',  '神吉', '雑事吉', '竪柱上梁修宅門戸碓磑廁納頭伐樹漁猟殯理斬草吉', 				 'けんふく ふねのり'	, nil					], # 30 甲午
	  ['三寶吉+', nil,    nil,	'壊垣吉',									 nil			, nil					], # 31 乙未
	  ['三寶吉+', '神吉', nil,	'祠祀壊垣解除除服伐樹葬斬草吉', 						 '木こり'		, nil					], # 32 丙申
	  ['三寶吉',  '神吉', nil,	'解除除服種蒔殯埋斬草吉',							 nil			, nil					], # 33 丁酉
	  [nil,       nil,    nil,	'漁猟種蒔吉',									 nil			, nil					], # 34 戊戌
	  ['三寶吉+', nil,    nil,	'療病吉',									 nil			, nil					], # 35 己亥
	  ['三寶吉',  '神吉', nil,	'塞穴斬草吉',									 nil			, nil					], # 36 庚子
	  ['三寶吉',  '神吉', nil,	'嫁娶出行吉',									 nil			, nil					], # 37 辛丑
	  ['三寶吉',  nil,    '雑事吉', '拜官結婚納徴療病壊垣破屋解除葬斬草吉', 					 'けんふく'		, nil					], # 38 壬寅
	  ['三寶吉',  '神吉', '雑事吉', '結婚納徴経絡納頭祀井修宅門戸碓磑廁斬草安床帳吉',				 'わたまし すゝはらひ'	, nil					], # 39 癸卯
	  ['三寶吉',  nil,    '雑事吉', '裁衣市買納財吉',								 nil			, nil					], # 40 甲辰
	  [nil,       nil,    nil,	'拜官冊授裁衣市買納財安床帳吉', 						 'いちたち'		, nil					], # 41 乙巳
	  ['三寶吉',  '神吉', '雑事吉', '納頭伐樹漁葬斬草吉',								 'かといて'		, nil					], # 42 丙午
	  [nil,       nil,    nil,	'不向産家',									 nil			, nil					], # 43 丁未
	  ['三寶吉',  '神吉', nil,	'壊垣破屋解除伐樹吉',								 '(みそさけつくり)'	, nil					], # 44 戊申
	  ['三寶吉',  '神吉', nil,	'出行竪柱上梁修宅門戸碓磑廁解除祀井吉', 					 nil			, nil					], # 45 己酉
	  ['三寶吉+', nil,    nil,	'漁猟市買納財吉',								 'たねまき'		, nil					], # 46 庚戌
	  ['三寶吉',  nil,    nil,	'療病上梁修宅門戸碓磑廁吉',							 '正月ことはしめ'	, nil					], # 47 辛亥
	  [nil,       nil,    nil,	'不打人不沐浴', 								 nil			, nil					], # 48 壬子
	  [nil,       nil,    nil,	'為解除不行病人家',								 nil			, nil					], # 49 癸丑
	  ['三寶吉',  nil,    nil,	'拜官結婚納徴療病解除除服掃舎壊垣斬草出行廁安床帳吉',				 '木こり'		, nil					], # 50 甲寅
	  ['三寶吉',  '神吉', '雑事吉', '拜官結婚納徴出行井碓磑廁斬草安床帳吉', 				       'ものたち 正月ことはしめ', nil					], # 51 乙卯
	  ['三寶吉+', nil,    '雑事吉', '嫁娶納婦吉',									 nil			, nil					], # 52 丙辰
	  [nil,       nil,    nil,	'拜官冊授裁衣市買納財吉',							 nil			, nil					], # 53 丁巳
	  ['三寶吉+', '神吉', '雑事吉', '納頭伐樹漁猟吉',								 'ふねのり'		, nil					], # 54 戊午
	  ['三寶吉',  '神吉', nil,	'壊垣破屋吉',									 'かといて'		, nil					], # 55 己未
	  [nil,       '神吉', nil,	'祠祀壊垣破屋伐樹解除除服葬吉', 						 'みそさけつくり'	, nil					], # 56 庚申
	  ['三寶吉',  '神吉', nil,	'謝土祀井解除除服吉',								 nil			, nil					], # 57 辛酉
	  ['三寶吉+', nil,    nil,	'漁猟種蒔吉',									 'たねまき'		, 'たねまき'				], # 58 壬戌
	  [nil,       nil,    nil,	'不見死人',									 nil			, nil					], # 59 癸亥
	]
      ]
    }
  end
end
